// To compile:
// csc Listing1_3cs.cs Listing1_2cs.cs IApplication.cs
// For demonstration, you can run this program on itself since it contains the IApplication interface:
// Listing1_3cs.exe Listing1_3cs.exe

using System;
using System.Reflection;

class UsingReflection
{
  public static void Main(String [] args)
  {
    if (args.Length < 1)
    {
      Console.WriteLine("You must specify an assembly name.");
      return;
    }

    Assembly a = null;

    try
    {
      // Load the assembly from a filename
      a = Assembly.LoadFrom(args[0]);
      Console.WriteLine("Assembly " + a.GetName());
    }
    catch (Exception e)
    {
      Console.WriteLine("Could not load the assembly: " + e.Message);
      return;
    }

    // Enumerate over the types in the assembly
    foreach (Type t in a.GetTypes())
    {
      try
      {
        KeywordAttribute [] attributes;

       // Check for the KeywordAttribute on the type
       attributes = (KeywordAttribute []) 
         t.GetCustomAttributes(typeof(KeywordAttribute), true);

       PrintKeywords(t.ToString(), attributes);

        // Enumerate over the members in the type
        foreach (MemberInfo member in t.GetMembers())
        {
         // Check for the KeywordAttribute on the member
         attributes = (KeywordAttribute []) 
           member.GetCustomAttributes(typeof(KeywordAttribute), true);

         PrintKeywords(t.ToString() + "." + member.Name, attributes);
        }
      }
      catch (Exception e)
      {
        Console.WriteLine(" Failed (" + e.GetType().ToString() + ").");
      }
    }
  }

  public static void PrintKeywords(string element, 
    KeywordAttribute [] attributes)
  {
    Console.WriteLine(">> " + element);

    foreach (KeywordAttribute a in attributes)
    {
     Console.WriteLine("   keyword = " + a.Keyword +
       " (" + a.Relevance + ")");
    }
  }
}