// To compile, first obtain an Interop Assembly for Microsoft Word 2002.  If a PIA is not yet
// available, you can run TLBIMP on msword.olb to create Word.dll.  Then run:
// csc WordProcessor.cs /r:System.Windows.Forms.dll /r:System.Drawing.dll /r:System.dll /r:Word.dll

using System;
using System.Drawing;
using System.Windows.Forms;

public class Form1 : Form
{
  // Visual controls
  private RichTextBox richTextBox1;
  private Button button1;
  private CheckBox checkBox1;
  private ContextMenu contextMenu1;

  // Required designer variable
  private System.ComponentModel.Container components = null;

  // The Word application object
  private Word.Application msWord;

  // Two fonts used to display normal words and misspelled words
  private Font normalFont = new Font("Times New Roman", 12);
  private Font errorFont = new Font("Times New Roman", 12, 
    FontStyle.Underline);

  // Objects that need to be passed by-reference when calling Word
  private object missing = Type.Missing;
  private object ignoreUpper;

  // Event handler used for the ContextMenu when 
  // the user clicks on spelling suggestions
  private EventHandler menuHandler;

  // Constructor
  public Form1()
  {
    // Required for Windows Form Designer support
    InitializeComponent();
    menuHandler = new System.EventHandler(this.Menu_Click);
  }

  // Called when the form is loading.  Initializes Microsoft Word.
  protected override void OnLoad(EventArgs e)
  {
    base.OnLoad(e);

    try
    {
      msWord = new Word.Application();

      // Call this in order for GetSpellingSuggestions to work later
      msWord.Documents.Add(ref missing, ref missing, ref missing, 
        ref missing);
    }
    catch (System.Runtime.InteropServices.COMException ex)
    {
      if ((uint)ex.ErrorCode == 0x80040154)
      {
        MessageBox.Show("This application requires Microsoft Word " +
          "to be installed for spelling functionality.  " +
          "Since Word can't be located, the spelling functionality " +
          "is disabled.", "Warning");
        button1.Enabled = false;
        checkBox1.Enabled = false;
      }
      else
      {
        MessageBox.Show("Unexpected initialization error.  " + 
          ex.Message + "\n\nDetails:\n" + ex.ToString(), 
          "Unexpected Initialization Error", MessageBoxButtons.OK, 
          MessageBoxIcon.Error);
        this.Close();
      }
    }
    catch (Exception ex)
    {
      MessageBox.Show("Unexpected initialization error.  " + ex.Message +
        "\n\nDetails:\n" + ex.ToString(), 
        "Unexpected Initialization Error", MessageBoxButtons.OK, 
        MessageBoxIcon.Error);
      this.Close();
    }
  }

  // Clean up any resources being used
  protected override void Dispose(bool disposing)
  {
    if (disposing)
    {
      if (components != null) 
      {
        components.Dispose();
      }
    }
    base.Dispose(disposing);
  }

  // Required method for Designer support
  private void InitializeComponent()
  {
    this.components = new System.ComponentModel.Container();
    this.contextMenu1 = new System.Windows.Forms.ContextMenu();
    this.richTextBox1 = new System.Windows.Forms.RichTextBox();
    this.button1 = new System.Windows.Forms.Button();
    this.checkBox1 = new System.Windows.Forms.CheckBox();
    this.richTextBox1.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
      | System.Windows.Forms.AnchorStyles.Left) 
      | System.Windows.Forms.AnchorStyles.Right);
    this.richTextBox1.Font = new System.Drawing.Font("Times New Roman", 12F);
    this.richTextBox1.Location = new System.Drawing.Point(8, 8);
    this.richTextBox1.Size = new System.Drawing.Size(344, 224);
    this.richTextBox1.TabIndex = 0;
    this.richTextBox1.Text = "";
    this.richTextBox1.MouseDown += new System.Windows.Forms.MouseEventHandler(this.richTextBox1_MouseDown);
    this.button1.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left);
    this.button1.Location = new System.Drawing.Point(8, 240);
    this.button1.Size = new System.Drawing.Size(112, 24);
    this.button1.TabIndex = 1;
    this.button1.Text = "Check Spelling";
    this.button1.Click += new System.EventHandler(this.button1_Click);
    this.checkBox1.Anchor = (System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left);
    this.checkBox1.Location = new System.Drawing.Point(136, 240);
    this.checkBox1.Size = new System.Drawing.Size(344, 24);
    this.checkBox1.TabIndex = 2;
    this.checkBox1.Text = "Ignore Words in UPPERCASE";
    this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
    this.ClientSize = new System.Drawing.Size(359, 272);
    this.Controls.AddRange(new System.Windows.Forms.Control[] {this.checkBox1, this.button1, this.richTextBox1});
    this.Text = "Word Processor";
  }

  // The main entry point for the application
  [STAThread]
  public static void Main()
  {
    Application.Run(new Form1());
  }

  // Checks the spelling of the word contained in the string argument.
  // Returns true if spelled correctly, false otherwise.
  // This method ignores words in all uppercase letters if checkBox1 
  // is checked.
  private bool CheckSpelling(string word)
  {
    ignoreUpper = checkBox1.Checked;

    // Pass a reference to Type.Missing for each 
    // by-reference optional parameter

    return msWord.CheckSpelling(word, // Word
      ref missing,                    // CustomDictionary
      ref ignoreUpper,                // IgnoreUppercase
      ref missing,                    // AlwaysSuggest
      ref missing,                    // CustomDictionary2
      ref missing,                    // CustomDictionary3
      ref missing,                    // CustomDictionary4
      ref missing,                    // CustomDictionary5
      ref missing,                    // CustomDictionary6
      ref missing,                    // CustomDictionary7
      ref missing,                    // CustomDictionary8
      ref missing,                    // CustomDictionary9
      ref missing);                   // CustomDictionary10
  }

  // Checks the spelling of the word contained in the string argument.
  // Returns a SpellingSuggestions collection, which is empty if the word
  // is spelled correctly.
  // This method ignores words in all uppercase letters if checkBox1
  // is checked.
  private Word.SpellingSuggestions GetSpellingSuggestions(string word)
  {
    ignoreUpper = checkBox1.Checked;

    // Pass a reference to Type.Missing for each 
    // by-reference optional parameter

    return msWord.GetSpellingSuggestions(word, // Word
      ref missing,                             // CustomDictionary
      ref ignoreUpper,                         // IgnoreUppercase
      ref missing,                             // MainDictionary
      ref missing,                             // SuggestionMode
      ref missing,                             // CustomDictionary2
      ref missing,                             // CustomDictionary3
      ref missing,                             // CustomDictionary4
      ref missing,                             // CustomDictionary5
      ref missing,                             // CustomDictionary6
      ref missing,                             // CustomDictionary7
      ref missing,                             // CustomDictionary8
      ref missing,                             // CustomDictionary9
      ref missing);                            // CustomDictionary10
  }

  // Called when the "Check Spelling" button is clicked.
  // Checks the spelling of each word and changes the font of 
  // each misspelled word.
  private void button1_Click(object sender, EventArgs e)
  {
    try
    {
      // Return all text to normal since 
      // underlined spaces might be left behind
      richTextBox1.SelectionStart = 0;
      richTextBox1.SelectionLength = richTextBox1.Text.Length;
      richTextBox1.SelectionFont = normalFont;

      // Beginning location in the RichTextBox of the next word to check
      int index = 0;

      // Enumerate over the collection of words obtained from String.Split
      foreach (string s in richTextBox1.Text.Split(null))
      {
        // Select the word
        richTextBox1.SelectionStart = index;
        richTextBox1.SelectionLength = s.Length;

        // Trim off any ending punctuation in the selected text
        while (richTextBox1.SelectionLength > 0 && 
          Char.IsPunctuation(
          richTextBox1.Text[index + richTextBox1.SelectionLength - 1]))
        {
          richTextBox1.SelectionLength--;
        }

        // Check the word's spelling
        if (!CheckSpelling(s))
        {
          // Mark as incorrect
          richTextBox1.SelectionFont = errorFont;
          richTextBox1.SelectionColor = Color.Red;
        }
        else
        {
          // Mark as correct
          richTextBox1.SelectionFont = normalFont;
          richTextBox1.SelectionColor = Color.Black;
        }

        // Update to point to the character after the current word
        index = index + s.Length + 1;
      }
    }
    catch (Exception ex)
    {
      MessageBox.Show("Unable to check spelling.  " + ex.Message + 
        "\n\nDetails:\n" + ex.ToString(), "Unable to Check Spelling", 
        MessageBoxButtons.OK, MessageBoxIcon.Error);
    }
  }

  // Called when the user clicks anywhere on the text.  If the user 
  // clicked the right mouse button, this determines the word underneath
  // the mouse pointer (if any) and presents a context menu of
  // spelling suggestions.
  private void richTextBox1_MouseDown(object sender, MouseEventArgs e)
  {
    try
    {
      if (e.Button == MouseButtons.Right)
      {
        // Get the location of the mouse pointer
        Point point = new Point(e.X, e.Y);

        // Find the index of the character underneath the mouse pointer
        int index = richTextBox1.GetCharIndexFromPosition(point);

        // Length of the word underneath the mouse pointer
        int length = 1;

        // If the character under the mouse pointer isn't whitespace,
        // determine what the word is and display spelling suggestions

        if (!Char.IsWhiteSpace(richTextBox1.Text[index]))
        {
          // Going backward from the index, 
          // figure out where the word begins
          while (index > 0 && !Char.IsWhiteSpace(
            richTextBox1.Text[index-1])) { index--; length++; }

          // Going forward, figure out where the word ends, 
          // making sure to not include punctuation except for apostrophes
          // (This works for English.)
          while (index + length < richTextBox1.Text.Length &&
            !Char.IsWhiteSpace(richTextBox1.Text[index + length]) &&
            (!Char.IsPunctuation(richTextBox1.Text[index + length]) || 
            richTextBox1.Text[index + length] == Char.Parse("'"))
            ) length++;

          // Now that we've found the entire word, select it
          richTextBox1.SelectionStart = index;
          richTextBox1.SelectionLength = length;

          // Clear the context menu in case 
          // there are items on it from last time
          contextMenu1.MenuItems.Clear();

          // Enumerate over the SpellingSuggestions collection 
          // returned by GetSpellingSuggestions
          foreach (Word.SpellingSuggestion s in 
            GetSpellingSuggestions(richTextBox1.SelectedText))
          {
            // Add the menu item with the suggestion text and the 
            // Menu_Click handler
            contextMenu1.MenuItems.Add(s.Name, menuHandler);
          }

          // Display special text if there are no spelling suggestions
          if (contextMenu1.MenuItems.Count == 0)
          {
            contextMenu1.MenuItems.Add("No suggestions.");
          }
          else
          {
            // Add two more items whenever there are spelling suggestions.
            // Since there is no event handler, nothing will happen when 
            // these are clicked.
            contextMenu1.MenuItems.Add("-");
            contextMenu1.MenuItems.Add("Don't change the spelling.");
          }

          // Now that the menu is ready, show it
          contextMenu1.Show(richTextBox1, point);
        }
      }
    }
    catch (Exception ex)
    {
      MessageBox.Show("Unable to give spelling suggestions.  " + 
        ex.Message + "\n\nDetails:\n" + ex.ToString(), 
        "Unable to Give Spelling Suggestions", MessageBoxButtons.OK, 
        MessageBoxIcon.Error);
    }
  }

  // Called when a spelling suggestion is clicked on the context menu.
  // Replaces the currently selected text with the text 
  // from the item clicked.
  private void Menu_Click(object sender, EventArgs e)
  {
    // The suggestion should be spelled correctly, 
    // so restore the font to normal
    richTextBox1.SelectionFont = normalFont;
    richTextBox1.SelectionColor = Color.Black;

    // Obtain the text from the MenuItem and replace the SelectedText
    richTextBox1.SelectedText = ((MenuItem)sender).Text;
  }
}