' To compile, first obtain an Interop Assembly for Microsoft Word 2002.  If a PIA is not yet
' available, you can run TLBIMP on msword.olb to create Word.dll.  Then run:
' vbc WordProcessor.vb /r:System.Windows.Forms.dll /r:System.Drawing.dll /r:System.dll /r:Word.dll

Imports System
Imports System.Drawing
Imports System.Windows.Forms
Imports Microsoft.VisualBasic

Public Class Form1
  Inherits Form

  ' Visual controls
  Private WithEvents richTextBox1 As RichTextBox
  Private WithEvents button1 As Button
  Private checkBox1 As CheckBox
  Private contextMenu1 As ContextMenu
  
  ' Required designer variable
  Private components As System.ComponentModel.Container = Nothing
  
  ' The Word application object
  Private msWord As Word.Application
  
  ' Two fonts used to display normal words and misspelled words
  Private normalFont As New Font("Times New Roman", 12)
  Private errorFont As New Font("Times New Roman", 12, FontStyle.Underline)
  
  ' Event handler used for the ContextMenu when 
  ' the user clicks on spelling suggestions
  Private menuHandler As EventHandler
   
  ' Constructor
  Public Sub New()
    ' Required for Windows Form Designer support
    InitializeComponent()
    menuHandler = AddressOf Menu_Click
  End Sub

  ' Called when the form is loading.  Initializes Microsoft Word.
  Protected Overrides Sub OnLoad(e As EventArgs)
    MyBase.OnLoad(e)

    Try
      msWord = New Word.Application()
 
      ' Call this in order for GetSpellingSuggestions to work later
       msWord.Documents.Add()
    Catch ex As System.Runtime.InteropServices.COMException
      If ex.ErrorCode = &H80040154 Then
        MessageBox.Show("This application requires Microsoft Word " & "to be installed for spelling functionality.  " & "Since Word can't be located, the spelling functionality " & "is disabled.", "Warning")
        button1.Enabled = False
        checkBox1.Enabled = False
      Else
        MessageBox.Show("Unexpected initialization error.  " & ex.Message & ControlChars.Lf & ControlChars.Lf & "Details:" & ControlChars.Lf & ex.ToString(), "Unexpected Initialization Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        Me.Close()
      End If
    Catch ex As Exception
      MessageBox.Show("Unexpected initialization error.  " & ex.Message & ControlChars.Lf & ControlChars.Lf & "Details:" & ControlChars.Lf & ex.ToString(), "Unexpected Initialization Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
      Me.Close()
    End Try
  End Sub

  ' Clean up any resources being used
  Protected Overrides Overloads Sub Dispose(disposing As Boolean)
    If disposing Then
      If Not (components Is Nothing) Then
        components.Dispose()
      End If
    End If
    MyBase.Dispose(disposing)
  End Sub

  ' Required method for Designer support
  Private Sub InitializeComponent()
    Me.components = New System.ComponentModel.Container()
    Me.contextMenu1 = New System.Windows.Forms.ContextMenu()
    Me.richTextBox1 = New System.Windows.Forms.RichTextBox()
    Me.button1 = New System.Windows.Forms.Button()
    Me.checkBox1 = New System.Windows.Forms.CheckBox()
    Me.richTextBox1.Anchor = System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left Or System.Windows.Forms.AnchorStyles.Right
    Me.richTextBox1.Font = New System.Drawing.Font("Times New Roman", 12F)
    Me.richTextBox1.Location = New System.Drawing.Point(8, 8)
    Me.richTextBox1.Size = New System.Drawing.Size(344, 224)
    Me.richTextBox1.TabIndex = 0
    Me.richTextBox1.Text = ""
    Me.button1.Anchor = System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left
    Me.button1.Location = New System.Drawing.Point(8, 240)
    Me.button1.Size = New System.Drawing.Size(112, 24)
    Me.button1.TabIndex = 1
    Me.button1.Text = "Check Spelling"
    Me.checkBox1.Anchor = System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left
    Me.checkBox1.Location = New System.Drawing.Point(136, 240)
    Me.checkBox1.Size = New System.Drawing.Size(344, 24)
    Me.checkBox1.TabIndex = 2
    Me.checkBox1.Text = "Ignore Words in UPPERCASE"
    Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
    Me.ClientSize = New System.Drawing.Size(359, 272)
    Me.Controls.AddRange(New System.Windows.Forms.Control() {Me.checkBox1, Me.button1, Me.richTextBox1})
    Me.Text = "Word Processor"
  End Sub

  ' The main entry point for the application
  Public Shared Sub Main()
    Application.Run(New Form1())
  End Sub

  ' Checks the spelling of the word contained in the string argument.
  ' Returns true if spelled correctly, false otherwise.
  ' This method ignores words in all uppercase letters if checkBox1 
  ' is checked.
  Private Function CheckSpelling(word As String) As Boolean
    CheckSpelling = msWord.CheckSpelling(word, , checkBox1.Checked)
  End Function

  ' Checks the spelling of the word contained in the string argument.
  ' Returns a SpellingSuggestions collection, which is empty if the word
  ' is spelled correctly.
  ' This method ignores words in all uppercase letters if checkBox1
  ' is checked.
  Private Function GetSpellingSuggestions(word As String) As Word.SpellingSuggestions
    GetSpellingSuggestions = msWord.GetSpellingSuggestions(word, , checkBox1.Checked)
  End Function

  ' Called when the "Check Spelling" button is clicked.
  ' Checks the spelling of each word and changes the font of 
  ' each misspelled word.
  Private Sub button1_Click(sender As Object, e As EventArgs) Handles button1.Click
    Try
      ' Return all text to normal since 
      ' underlined spaces might be left behind
      richTextBox1.SelectionStart = 0
      richTextBox1.SelectionLength = richTextBox1.Text.Length
      richTextBox1.SelectionFont = normalFont
 
      ' Beginning location in the RichTextBox of the next word to check
      Dim index As Integer = 0
 
      ' Enumerate over the collection of words obtained from String.Split
      Dim s As String
      For Each s In richTextBox1.Text.Split(Nothing)
        ' Select the word
        richTextBox1.SelectionStart = index
        richTextBox1.SelectionLength = s.Length
            
        ' Trim off any ending punctuation in the selected text
        While richTextBox1.SelectionLength > 0 And Char.IsPunctuation(richTextBox1.Text.Chars(index + richTextBox1.SelectionLength - 1))
          richTextBox1.SelectionLength -= 1
        End While
 
        ' Check the word's spelling
        If Not CheckSpelling(s) Then
          ' Mark as incorrect
          richTextBox1.SelectionFont = errorFont
          richTextBox1.SelectionColor = Color.Red
        Else
          ' Mark as correct
          richTextBox1.SelectionFont = normalFont
          richTextBox1.SelectionColor = Color.Black
        End If
          
        ' Update to point to the character after the current word
        index = index + s.Length + 1
      Next
    Catch ex As Exception
      MessageBox.Show("Unable to check spelling.  " & ex.Message & ControlChars.Lf & ControlChars.Lf & "Details:" & ControlChars.Lf & ex.ToString(), "Unable to Check Spelling", MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try
  End Sub

  ' Called when the user clicks anywhere on the text.  If the user 
  ' clicked the right mouse button, this determines the word underneath
  ' the mouse pointer (if any) and presents a context menu of
  ' spelling suggestions.
  Private Sub richTextBox1_MouseDown(sender As Object, e As MouseEventArgs) Handles richTextBox1.MouseDown
    Try
      If e.Button = MouseButtons.Right Then
        ' Get the location of the mouse pointer
        Dim point As New Point(e.X, e.Y)
            
        ' Find the index of the character underneath the mouse pointer
        Dim index As Integer = richTextBox1.GetCharIndexFromPosition(point)
            
        ' Length of the word underneath the mouse pointer
        Dim length As Integer = 1
            
        ' If the character under the mouse pointer isn't whitespace,
        ' determine what the word is and display spelling suggestions
        If Not Char.IsWhiteSpace(richTextBox1.Text.Chars(index)) Then
          ' Going backward from the index, 
          ' figure out where the word begins
          While index > 0 AndAlso Not Char.IsWhiteSpace(richTextBox1.Text.Chars(index - 1))
            index -= 1
            length += 1
          End While 

          ' Going forward, figure out where the word ends, 
          ' making sure to not include punctuation except for apostrophes
          ' (This works for English.)
          While index + length < richTextBox1.Text.Length AndAlso Not Char.IsWhiteSpace(richTextBox1.Text.Chars(index + length)) AndAlso _
            (Not Char.IsPunctuation(richTextBox1.Text.Chars(index + length)) Or richTextBox1.Text.Chars(index + length) = Char.Parse("'"))
            length += 1
          End While 

          ' Now that we've found the entire word, select it
          richTextBox1.SelectionStart = index
          richTextBox1.SelectionLength = length
               
          ' Clear the context menu in case 
          ' there are items on it from last time
          contextMenu1.MenuItems.Clear()
               
          ' Enumerate over the SpellingSuggestions collection 
          ' returned by GetSpellingSuggestions
          Dim s As Word.SpellingSuggestion
          For Each s In GetSpellingSuggestions(richTextBox1.SelectedText)
            ' Add the menu item with the suggestion text and the 
            ' Menu_Click handler
            contextMenu1.MenuItems.Add(s.Name, menuHandler)
          Next
           
          ' Display special text if there are no spelling suggestions
          If contextMenu1.MenuItems.Count = 0 Then
            contextMenu1.MenuItems.Add("No suggestions.")
          Else
            ' Add two more items whenever there are spelling suggestions.
            ' Since there is no event handler, nothing will happen when 
            ' these are clicked.
            contextMenu1.MenuItems.Add("-")
            contextMenu1.MenuItems.Add("Don't change the spelling.")
          End If
  
          ' Now that the menu is ready, show it
          contextMenu1.Show(richTextBox1, point)
        End If
      End If
    Catch ex As Exception
      MessageBox.Show("Unable to give spelling suggestions.  " & ex.Message & ControlChars.Lf & ControlChars.Lf & "Details:" & ControlChars.Lf & ex.ToString(), "Unable to Give Spelling Suggestions", MessageBoxButtons.OK, MessageBoxIcon.Error)
    End Try
  End Sub
   
  ' Called when a spelling suggestion is clicked on the context menu.
  ' Replaces the currently selected text with the text 
  ' from the item clicked.
  Private Sub Menu_Click(sender As Object, e As EventArgs)
    ' The suggestion should be spelled correctly, 
    ' so restore the font to normal
    richTextBox1.SelectionFont = normalFont
    richTextBox1.SelectionColor = Color.Black

    ' Obtain the text from the MenuItem and replace the SelectedText
    richTextBox1.SelectedText = CType(sender, MenuItem).Text
  End Sub

End Class