// Before compiling, obtain an Interop Assembly for shdocvw.dll.  If a PIA isn't available,
// run TLBIMP on it.  Then run:
// csc Listing5_7cs.cs /r:SHDocVw.dll

using System;
using SHDocVw;
using System.Runtime.InteropServices;

public class BrowserListener : DWebBrowserEvents2
{
  private UCOMIConnectionPoint icp;  // The connection point
  private int cookie = -1;           // The cookie for the connection

  public BrowserListener()
  {
    InternetExplorer ie = new InternetExplorer();

    // Call QueryInterface for IConnectionPointContainer
    UCOMIConnectionPointContainer icpc = (UCOMIConnectionPointContainer)ie;

    // Find the connection point for the 
    // DWebBrowserEvents2 source interface
    Guid g = typeof(DWebBrowserEvents2).GUID;
    icpc.FindConnectionPoint(ref g, out icp);

    // Pass a pointer to the host to the connection point
    icp.Advise(this, out cookie);

    // Show the browser
    ie.Visible = true;
    ie.GoHome();
  }

  ~BrowserListener()
  {
    // End the connection
    if (cookie != -1) icp.Unadvise(cookie);
  }

  // Event handlers for all of the source interface's methods

  public void DownloadBegin()
  {
    Console.WriteLine("DownloadBegin");
  }

  public void NavigateComplete2(object pDisp, ref object URL)
  {
    Console.WriteLine("NavigateComplete2: " + URL);
  }

  public void OnQuit()
  {
    Console.WriteLine("OnQuit");

    // End the connection
    icp.Unadvise(cookie);
    cookie = -1;

    Environment.Exit(0);
  }

  public void OnStatusBar(bool StatusBar)
  {
    Console.WriteLine("OnStatusBar: " + StatusBar);
  }

  public void BeforeNavigate2(object pDisp, ref object URL, ref object Flags, ref object TargetFrameName, ref object PostData, ref object Headers, ref bool Cancel)
  {
    Console.WriteLine("BeforeNavigate2: " + URL);
  }

  public void ClientToHostWindow(ref int CX, ref int CY)
  {
    Console.WriteLine("ClientToHostWindow: (" + CX + ", " + CY + ")");
  }

  public void CommandStateChange(int Command, bool Enable)
  {
    Console.WriteLine("CommandStateChange: command=" + Command + ", enable=" + Enable);
  }

  public void DocumentComplete(object pDisp, ref object URL)
  {
    Console.WriteLine("DocumentComplete: " + URL);
  }

  public void DownloadComplete()
  {
    Console.WriteLine("DownloadComplete");
  }

  public void FileDownload(ref bool Cancel)
  {
    Console.WriteLine("FileDownload: cancel=" + Cancel);
  }

  public void NavigateError(object pDisp, ref object URL, ref object Frame, ref object StatusCode, ref bool Cancel)
  {
    Console.WriteLine("NavigateError: " + URL);
  }

  public void NewWindow2(ref object ppDisp, ref bool Cancel)
  {
    Console.WriteLine("NewWindow2: cancel=" + Cancel);
  }

  public void OnFullScreen(bool FullScreen)
  {
    Console.WriteLine("OnFullScreen: " + FullScreen);
  }

  public void OnMenuBar(bool MenuBar)
  {
    Console.WriteLine("OnMenuBar: " + MenuBar);
  }

  public void OnTheaterMode(bool TheaterMode)
  {
    Console.WriteLine("OnTheaterMode: " + TheaterMode);
  }

  public void OnToolBar(bool ToolBar)
  {
    Console.WriteLine("OnToolBar: " + ToolBar);
  }

  public void OnVisible(bool Visible)
  {
    Console.WriteLine("OnVisible: " + Visible);
  }

  public void PrintTemplateInstantiation(object pDisp)
  {
    Console.WriteLine("PrintTemplateInstantiation");
  }

  public void PrintTemplateTeardown(object pDisp)
  {
    Console.WriteLine("PrintTemplateTeardown");
  }

  public void PrivacyImpactedStateChange(bool bImpacted)
  {
    Console.WriteLine("PrivacyImpactedStateChange: " + bImpacted);
  }

  public void ProgressChange(int Progress, int ProgressMax)
  {
    Console.WriteLine("ProgressChange: " + Progress + " out of " + ProgressMax);
  }

  public void PropertyChange(string szProperty)
  {
    Console.WriteLine("PropertyChange: " + szProperty);
  }

  public void SetSecureLockIcon(int SecureLockIcon)
  {
    Console.WriteLine("SetSecureLockIcon: " + SecureLockIcon);
  }

  public void StatusTextChange(string Text)
  {
    Console.WriteLine("StatusTextChange: " + Text);
  }

  public void TitleChange(string Text)
  {
    Console.WriteLine("TitleChange: " + Text);
  }

  public void UpdatePageStatus(object pDisp, ref object nPage, ref object fDone)
  {
    Console.WriteLine("UpdatePageStatus: " + nPage);
  }

  public void WindowClosing(bool IsChildWindow, ref bool Cancel)
  {
    Console.WriteLine("WindowClosing: childWindow=" + IsChildWindow + ", cancel=" + Cancel);
  }

  public void WindowSetHeight(int Height)
  {
    Console.WriteLine("WindowSetHeight: " + Height);
  }

  public void WindowSetLeft(int Left)
  {
    Console.WriteLine("WindowSetLeft: " + Left);
  }

  public void WindowSetResizable(bool Resizable)
  {
    Console.WriteLine("WindowSetResizable: " + Resizable);
  }

  public void WindowSetTop(int Top)
  {
    Console.WriteLine("WindowSetTop: " + Top);
  }

  public void WindowSetWidth(int Width)
  {
    Console.WriteLine("WindowSetWidth: " + Width);
  }

  public static void Main()
  {
    BrowserListener host = new BrowserListener();

    // Keep the program running while Internet Explorer is open
    Console.WriteLine("*** Press Enter to quit ***");
    Console.Read();
  }
}