// Before compiling, compile the VB6 project Project6_1.vbp to create Project6_1.dll.
// Then run "tlbimp Project6_1.dll" to create ComObject.dll.  Then run "ildasm ComObject.dll /out:ComObject.il"
// and change both occurrences of:
//   [in][out] object&  marshal( struct) v
// to:
//   native int v
// in ComObject.il.  Then run "ilasm /dll ComObject.il" to create the final ComObject.dll. Then run:
// csc Listing6_1.cs /r:ComObject.dll

using System;
using System.Reflection;
using System.Runtime.InteropServices;
using ComObject;

public struct Variant
{
  public ushort vt;
  public ushort wReserved1;
  public ushort wReserved2;
  public ushort wReserved3;
  public IntPtr pvRecord;
  public IntPtr pRecInfo;
}

[
  ComImport, 
  Guid("0000002F-0000-0000-C000-000000000046"), 
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
public interface IRecordInfo
{
  void RecordInit(IntPtr pvNew);
  void RecordClear(IntPtr pvExisting);
  void RecordCopy(IntPtr pvExisting, IntPtr pvNew);
  Guid GetGuid();
  string GetName();
  uint GetSize();
  UCOMITypeInfo GetTypeInfo();
  object GetField(IntPtr pvData, 
    [MarshalAs(UnmanagedType.LPWStr)] string szFieldName);
  IntPtr GetFieldNoCopy(IntPtr pvData, [MarshalAs(UnmanagedType.LPWStr)] 
    string szFieldName, ref object pvarField);
  void PutField(uint wFlags, IntPtr pvData, [MarshalAs(
    UnmanagedType.LPWStr)] string szFieldName, [In] ref object pvarField);
  void PutFieldNoCopy(uint wFlags, IntPtr pvData, [MarshalAs(
    UnmanagedType.LPWStr)] string szFieldName, [In] ref object pvarField);
  void GetFieldNames(IntPtr pcNames, [In, Out] string [] rgBstrNames);
  [PreserveSig]
  bool IsMatchingType(IRecordInfo pRecordInfo);
  [PreserveSig]
  IntPtr RecordCreate();
  void RecordCreateCopy(IntPtr pvSource, ref IntPtr ppvDest);
  void RecordDestroy(IntPtr pvRecord);
}

public class VT_RECORDWorkaround
{
  [DllImport("oleaut32.dll", PreserveSig=false)]
  private static extern IntPtr 
    GetRecordInfoFromTypeInfo(IntPtr pTypeInfo);

  public static void Main()
  {
    Class1 obj = new Class1();

    Variant v = new Variant();
    
    // Create an IRecordInfo instance
    IntPtr ptr = GetRecordInfoFromTypeInfo(
      Marshal.GetITypeInfoForType(typeof(RECT)));
    IRecordInfo info = (IRecordInfo)Marshal.GetObjectForIUnknown(ptr);

    // Initialize the Variant
    v.vt = (ushort)VarEnum.VT_RECORD; // VT_RECORD = 36
    v.pvRecord = info.RecordCreate();
    v.pRecInfo = ptr;

    GCHandle handle = new GCHandle();

    try
    {
      // Pin the boxed value type just so we can get the address
      handle = GCHandle.Alloc(v, GCHandleType.Pinned);

      // Call the COM object's method expecting a pointer to a Variant
      obj.FillRect(handle.AddrOfPinnedObject());
    }
    finally
    {
      if (handle.IsAllocated) handle.Free();
    }
 
    // Convert the VT_RECORD Variant to the wrapped value type
    object o = new RECT();
    VT_RECORDToStructure(v, ref o);
    RECT r = (RECT)o;

    Console.WriteLine(
      r.Left + " " + r.Top + " " + r.Right + " " + r.Bottom);
  }

  public static void VT_RECORDToStructure(Variant v, ref Object o)	
  {
    if (v.vt != (ushort)VarEnum.VT_RECORD)
      throw new ArgumentException("Incorrect VARIANT type");
    
    IRecordInfo info = 
      (IRecordInfo)Marshal.GetObjectForIUnknown(v.pRecInfo);

    if (info.GetName() != o.GetType().Name)
    {
      throw new ArgumentException("Variant name '" + info.GetName() +
        "' doesn't match object name '" + o.GetType().Name + "'.");
    }

    // Copy every field's data, thanks to reflection
    foreach (FieldInfo fi in o.GetType().GetFields()) 
    { 
      fi.SetValue(o, info.GetField(v.pvRecord, fi.Name));
    }
  }
}