#define _WIN32_DCOM
#include <stdio.h>
#include <wtypes.h>

// To run this code, first compile StringValidator.cs or StringValidator.vb.
// Then run the following:
//   gacutil -i StringValidator.dll
//   regasm /tlb StringValidator.dll

// Reference the type library for the StringValidator assembly
#import "StringValidator.tlb" no_namespace named_guids raw_interfaces_only

int main(int argc, char* argv[])
{
  IUnknown* pUnk = NULL;
  IDispatch* pDisp = NULL;
  HRESULT hresult;

  // Initialize COM
  hresult = CoInitializeEx(NULL, COINIT_MULTITHREADED);
  if (FAILED(hresult)) 
  {
    printf("ERROR: Cannot initialize COM: 0x%x\n", hresult);
    return -1;
  }

  // Instantiate the .NET StringValidator class
  hresult = CoCreateInstance(CLSID_StringValidator, NULL, 
    CLSCTX_INPROC_SERVER, IID_IUnknown, (void **)&pUnk);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot create object: 0x%x\n", hresult);
    return -1;
  }

  // Get the IDispatch interface
  hresult = pUnk->QueryInterface(IID_IDispatch, (void**)&pDisp);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot obtain the IDispatch interface pointer: 0x%x\n",
      hresult);
    pUnk->Release();
    return -1;
  }

  pUnk->Release();

  // Get the DISPID for the IsZipCode method
  OLECHAR* name = L"IsZipCode";
  DISPID dispid;
  hresult = pDisp->GetIDsOfNames(IID_NULL, &name, 1, GetUserDefaultLCID(),
    &dispid);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: GetIDsOfNames failed: 0x%x\n", hresult);
    pDisp->Release();
    return -1;
  }

  // Initialize the IsZipCode parameter
  VARIANT args[1];
  VariantInit(&args[0]);
  args[0].vt = VT_BSTR;
  args[0].bstrVal = SysAllocString(L"90210");
  DISPPARAMS params = { args, NULL, 1, 0 };

  // Invoke the IsZipCode method
  VARIANT result;
  VariantInit(&result);

  hresult = pDisp->Invoke(dispid, IID_NULL, GetUserDefaultLCID(), 
    DISPATCH_METHOD, &params, &result, NULL, NULL);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Invoke failed: 0x%x\n", hresult);
  }
  else
  {
    if (result.boolVal == VARIANT_TRUE)
      printf("The input string is a valid zip code.\n");
    else
      printf("The input string is not a valid zip code.\n");
  }

  // Free the string and release the IDispatch pointer
  VariantClear(&args[0]);
  pDisp->Release();

  CoUninitialize();
  return 0;
}