#define _WIN32_DCOM
#include <stdio.h>
#include <wtypes.h>

// Reference the type library for mscorlib
#import "c:\\Windows\\Microsoft.NET\\Framework\\v1.0.3705\\mscorlib.tlb" no_namespace named_guids raw_interfaces_only

int main(int argc, char* argv[])
{
  IUnknown* pUnk = NULL;
  IDispatch* pDisp = NULL;
  HRESULT hresult;

  // Initialize COM
  hresult = CoInitializeEx(NULL, COINIT_MULTITHREADED);
  if (FAILED(hresult)) 
  {
    printf("ERROR: Cannot initialize COM: 0x%x\n", hresult);
    return -1;
  }

  // Instantiate the System.Collections.Stack class
  hresult = CoCreateInstance(CLSID_Stack, NULL, CLSCTX_INPROC_SERVER, 
     IID_IUnknown, (void **)&pUnk);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot create object: 0x%x\n", hresult);
    return -1;
  }

  // Get the IDispatch interface
  hresult = pUnk->QueryInterface(IID_IDispatch, (void**)&pDisp);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot obtain IDispatch pointer: 0x%x\n", hresult);
    pUnk->Release();
    return -1;
  }

  pUnk->Release();

  // Get the DISPID for the Pop method
  OLECHAR* name = L"Pop";
  DISPID dispid;
  hresult = pDisp->GetIDsOfNames(IID_NULL, &name, 1,
    GetUserDefaultLCID(), &dispid);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: GetIDsOfNames failed: 0x%x\n", hresult);
    pDisp->Release();
    return -1;
  }

  // Invoke the Pop method
  VARIANT result;
  VariantInit(&result);
  DISPPARAMS params = { NULL, NULL, 0, 0 };
  EXCEPINFO excepInfo;

  hresult = pDisp->Invoke(dispid, IID_NULL, GetUserDefaultLCID(),
    DISPATCH_METHOD, &params, &result, &excepInfo, NULL);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Invoke failed:          0x%x\n", hresult);
    if (hresult == DISP_E_EXCEPTION)
    {
      printf("HRESULT returned by Stack.Pop: 0x%x\n", excepInfo.scode);
      printf("Exception Source:              %S\n", excepInfo.bstrSource);
      printf("Exception Message:             %S\n", 
        excepInfo.bstrDescription);
      printf("HelpFile:                      %S\n", 
        excepInfo.bstrHelpFile);
      printf("HelpContext:                   %d\n", 
        excepInfo.dwHelpContext);

      if (excepInfo.bstrSource != NULL) 
         SysFreeString(excepInfo.bstrSource);
      if (excepInfo.bstrDescription != NULL) 
        SysFreeString(excepInfo.bstrDescription);
      if (excepInfo.bstrHelpFile != NULL) 
        SysFreeString(excepInfo.bstrHelpFile);
    }
  }

  // Release the IDispatch pointer
  pDisp->Release();

  CoUninitialize();
  return 0;
}