#define _WIN32_DCOM
#include <stdio.h>
#include <wtypes.h>

#import "c:\\Windows\\Microsoft.NET\\Framework\\v1.0.3705\\mscorlib.tlb" no_namespace named_guids raw_interfaces_only

int main(int argc, char* argv[])
{
  IUnknown* pUnk = NULL;
  IList* pList = NULL;
  ISupportErrorInfo* pSuppErrInfo = NULL;
  IErrorInfo* pErrInfo = NULL;
  _Exception* pException = NULL;
  _Type* pType = NULL;
  HRESULT hresult;

  // Initialize COM
  hresult = CoInitializeEx(NULL, COINIT_MULTITHREADED);
  if (FAILED(hresult)) 
  {
    printf("ERROR: Cannot initialize COM: 0x%x\n", hresult);
    return -1;
  }

  // Instantiate the System.Collections.ArrayList class
  hresult = CoCreateInstance(CLSID_ArrayList, NULL, CLSCTX_INPROC_SERVER, 
    IID_IUnknown, (void **)&pUnk);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot create object: 0x%x\n", hresult);
    return -1;
  }

  // Get the IList interface
  hresult = pUnk->QueryInterface(IID_IList, (void**)&pList);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot obtain the IList interface pointer: 0x%x\n",
      hresult);
    pUnk->Release();
    return -1;
  }

  hresult = pList->RemoveAt(1);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: RemoveAt failed: 0x%x\n\n", hresult);

    // Check if the object supports extended error information
    hresult = pList->QueryInterface(IID_ISupportErrorInfo,
      (void**)&pSuppErrInfo);
    if (SUCCEEDED(hresult))
    {
      hresult = pSuppErrInfo->InterfaceSupportsErrorInfo(IID_IList);
      if (hresult == S_OK) 
      {
        // Attempt to get the IErrorInfo pointer
        hresult = GetErrorInfo(0, &pErrInfo);
        if (SUCCEEDED(hresult))
        {
          BSTR source = NULL;
          BSTR description = NULL;
          BSTR helpFile = NULL;
          unsigned long helpContext;

          hresult = pErrInfo->GetSource(&source);
          if (SUCCEEDED(hresult)) printf("Source:         %S\n\n", 
            source);

          hresult = pErrInfo->GetDescription(&description);
          if (SUCCEEDED(hresult)) printf("Description:    %S\n\n",
            description);

          hresult = pErrInfo->GetHelpFile(&helpFile);
          if (SUCCEEDED(hresult)) printf("HelpFile:       %S\n\n", 
            helpFile);

          hresult = pErrInfo->GetHelpContext(&helpContext);
          if (SUCCEEDED(hresult)) printf("HelpContext:    %d\n\n",
            helpContext);
  
          if (source != NULL) SysFreeString(source);
          if (description != NULL) SysFreeString(description);
          if (helpFile != NULL) SysFreeString(helpFile);

          // Call members of the .NET exception object
          BSTR stackTrace = NULL;
          BSTR toString = NULL;
          BSTR exceptionTypeName = NULL;

          hresult = pErrInfo->QueryInterface(IID__Exception,
            (void**)&pException);
          if (SUCCEEDED(hresult))
          {
            hresult = pException->get_StackTrace(&stackTrace);
            if (SUCCEEDED(hresult)) printf("Stack Trace:    %S\n\n", 
              stackTrace);

            hresult = pException->get_ToString(&toString);
            if (SUCCEEDED(hresult)) printf("ToString:       %S\n\n", 
              toString);

            hresult = pException->GetType(&pType);
            if (SUCCEEDED(hresult))
            {
              hresult = pType->get_ToString(&exceptionTypeName);
              if (SUCCEEDED(hresult)) printf("Exception Type: %S\n\n", 
                exceptionTypeName);
            }

            if (toString != NULL) SysFreeString(toString);
            if (stackTrace != NULL) SysFreeString(stackTrace);
            if (exceptionTypeName != NULL) 
              SysFreeString(exceptionTypeName);
          }
        }
      }
    }
  }

  // Release interface pointers
  if (pUnk != NULL) pUnk->Release();
  if (pList != NULL) pList->Release();
  if (pSuppErrInfo != NULL) pSuppErrInfo->Release();
  if (pErrInfo != NULL) pErrInfo->Release();
  if (pException != NULL) pException->Release();
  if (pType != NULL) pType->Release();

  CoUninitialize();
  return 0;
}