// To compile:
// csc /t:library LabeledTextBox.cs /r:System.Windows.Forms.dll /r:System.Drawing.dll /r:System.dll

using System;
using System.Security;
using Microsoft.Win32;
using System.Windows.Forms;
using System.Runtime.InteropServices;

[assembly:AllowPartiallyTrustedCallersAttribute]

public class LabeledTextBox : UserControl
{
  private System.Windows.Forms.TextBox textBox1;
  private System.Windows.Forms.Label label1;
  private System.ComponentModel.Container components = null;

  // Custom Registration Function
  [ComRegisterFunction]
  private static void ComRegisterFunction(Type t)
  {
    RegistryKey key = Registry.ClassesRoot.OpenSubKey("CLSID\\" +
      t.GUID.ToString("B"), true);
    // Step 1
    key.CreateSubKey("Control");
    // Step 2
    key.CreateSubKey(
      "Implemented Categories\\{40FC6ED4-2438-11CF-A3DB-080036F12502}");
    // Step 3
    key.CreateSubKey("MiscStatus").SetValue("", "131457");
    // Step 4
    key.CreateSubKey("TypeLib").SetValue("",
      Marshal.GetTypeLibGuidForAssembly(t.Assembly).ToString("B"));
    // Step 5
    string version = t.Assembly.GetName().Version.Major.ToString() +
      "." + t.Assembly.GetName().Version.Minor.ToString();
    if (version == "0.0") version = "1.0";
    key.CreateSubKey("Version").SetValue("", version);
    key.Close();
  }

  // Custom Unregistration Function
  [ComUnregisterFunction]
  private static void ComUnregisterFunction(Type t)
  {
    try 
    {
      Registry.ClassesRoot.DeleteSubKeyTree(
        "CLSID\\" + t.GUID.ToString("B"));
    }
    // Ignore exception thrown if key doesn't exist
    catch (ArgumentException) {} 
  }

  public LabeledTextBox()
  {
    // This call is required by the Windows.Forms Form Designer
    InitializeComponent();
  }

  public string LabelText
  {
    get { return label1.Text; }
    set { label1.Text = value; }
  }

  public string TextBoxText
  {
    get { return textBox1.Text; }
    set { textBox1.Text = value; }
  }

  // Clean up any resources being used
  protected override void Dispose(bool disposing)
  {
    if (disposing)
    {
      if (components != null)
        components.Dispose();
    }
    base.Dispose(disposing);
  }

  // Required method for Designer support
  private void InitializeComponent()
  {
    this.textBox1 = new System.Windows.Forms.TextBox();
    this.label1 = new System.Windows.Forms.Label();
    this.SuspendLayout();

    this.textBox1.Anchor = ((System.Windows.Forms.AnchorStyles.Top |
      System.Windows.Forms.AnchorStyles.Bottom) 
      | System.Windows.Forms.AnchorStyles.Right);
    this.textBox1.Location = new System.Drawing.Point(112, 0);
    this.textBox1.Multiline = true;
    this.textBox1.Name = "textBox1";
    this.textBox1.Size = new System.Drawing.Size(144, 20);
    this.textBox1.TabIndex = 0;
    this.textBox1.Text = "TextBox";

    this.label1.Anchor = (((System.Windows.Forms.AnchorStyles.Top |
      System.Windows.Forms.AnchorStyles.Bottom) 
      | System.Windows.Forms.AnchorStyles.Left) 
      | System.Windows.Forms.AnchorStyles.Right);
    this.label1.Name = "label1";
    this.label1.Size = new System.Drawing.Size(112, 20);
    this.label1.TabIndex = 1;
    this.label1.Text = "Label";
    this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleRight;

    this.Controls.AddRange(new System.Windows.Forms.Control[] {this.label1, this.textBox1});
    this.Name = "LabeledTextBox";
    this.Size = new System.Drawing.Size(256, 20);
    this.ResumeLayout(false);
  }
}