#define _WIN32_DCOM
#include <stdio.h>
#include <wtypes.h>

// Reference the two necessary type libraries
#import "C:\\Windows\\Microsoft.NET\\Framework\\v1.0.3705\\mscoree.tlb" no_namespace named_guids raw_interfaces_only
#import "C:\\Windows\\Microsoft.NET\\Framework\\v1.0.3705\\mscorlib.tlb" no_namespace named_guids raw_interfaces_only

IUnknown* pUnk = NULL;
IUnknown* pUnk2 = NULL;
ICorRuntimeHost* pHost = NULL;
_AppDomain* pDomain = NULL;
_ObjectHandle* pHandle = NULL;
IDispatch* pDisp = NULL;

BSTR asmName;
BSTR typeName;
VARIANT arrayList;
VARIANT param;

void Cleanup()
{
  pHost->Stop();
  if (pUnk) pUnk->Release();
  if (pUnk2) pUnk2->Release();
  if (pDisp) pDisp->Release();
  if (pHandle) pHandle->Release();
  if (pDomain) pDomain->Release();
  if (pHost) pHost->Release();
  if (asmName) SysFreeString(asmName);
  if (typeName) SysFreeString(typeName);
  VariantClear(&param);
  VariantClear(&arrayList);
  CoUninitialize();
};

int main(int argc, char* argv[])
{
  HRESULT hresult;

  // Initialize COM
  hresult = CoInitializeEx(NULL, COINIT_MULTITHREADED);
  if (FAILED(hresult)) 
  {
    printf("ERROR: Cannot initialize COM: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Initialize the CLR
  hresult = CoCreateInstance(CLSID_CorRuntimeHost, NULL, 
    CLSCTX_INPROC_SERVER, IID_IUnknown, (void **)&pUnk);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot create host object: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Get the ICorRuntimeHost interface
  hresult = pUnk->QueryInterface(IID_ICorRuntimeHost, (void**)&pHost);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot get ICorRuntimeHost interface pointer: 0x%x\n",
      hresult);
    Cleanup();
    return -1;
  }

  // Start the host
  hresult = pHost->Start();
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot start host: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Get the default domain
  hresult = pHost->GetDefaultDomain(&pUnk2);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot get default domain: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Get the _AppDomain interface
  hresult = pUnk2->QueryInterface(IID__AppDomain, (void**)&pDomain);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot get _AppDomain interface pointer: 0x%x\n",
      hresult);
    Cleanup();
    return -1;
  }

  // Strings for CreateInstance_3
  asmName = SysAllocString(L"mscorlib");
  typeName = SysAllocString(L"System.Collections.ArrayList");

  // Create a 1D array with one integer element
  SAFEARRAY* psa = SafeArrayCreateVector(VT_VARIANT, 0, 1);
  VariantInit(&param);
  param.vt = VT_I4;
  param.lVal = 128;
  LONG index = 0;

  hresult = SafeArrayPutElement(psa, &index, &param);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot set SAFEARRAY element: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Create an instance of ArrayList using a parameterized constructor
  hresult = pDomain->CreateInstance_3(asmName, typeName, VARIANT_TRUE,
    BindingFlags_Default, NULL, psa, NULL, NULL, NULL, &pHandle);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Cannot create instance: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Unwrap the ArrayList instance inside the ObjectHandle
  VariantInit(&arrayList);
  hresult = pHandle->Unwrap(&arrayList);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Could not unwrap object handle: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Get the IDispatch interface so we can call the Capacity property
  hresult = arrayList.punkVal->QueryInterface(IID_IDispatch,
    (void**)&pDisp);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Could not get IDispatch interface pointer: 0x%x\n",
      hresult);
    Cleanup();
    return -1;
  }

  // Get the DISPID for the Capacity property
  OLECHAR* name = L"Capacity";
  DISPID dispid;
  hresult = pDisp->GetIDsOfNames(IID_NULL, &name, 1, GetUserDefaultLCID(),
    &dispid);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: GetIDsOfNames failed: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  // Invoke the Capacity property
  VARIANT result;
  VariantInit(&result);
  DISPPARAMS params = { NULL, NULL, 0, 0 };

  hresult = pDisp->Invoke(dispid, IID_NULL, GetUserDefaultLCID(), 
    DISPATCH_PROPERTYGET, &params, &result, NULL, NULL);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Invoke failed: 0x%x\n", hresult);
    Cleanup();
    return -1;
  }

  printf("ArrayList Capacity: %d\n", result.lVal);

  Cleanup();
  return 0;
};