#define _WIN32_DCOM
#include <stdio.h>
#include <wtypes.h>

#import "C:\\Windows\\Microsoft.NET\\Framework\\v1.0.3705\\mscorlib.tlb" no_namespace named_guids raw_interfaces_only

HRESULT InvokeStaticMember(BSTR typeName, BSTR memberName, 
  BindingFlags memberType, SAFEARRAY* parameters, VARIANT* retVal)
{
  HRESULT hresult;
  IUnknown* pUnk = NULL;
  _Object* pObj = NULL;
  _Type* pType = NULL;
  _Type* pTypeOfType = NULL;
  _Type* pDesiredType = NULL;
  VARIANT typeNameParam;
  VARIANT getTypeRetVal;
  VARIANT nullObject;
  SAFEARRAY* psa;
  LONG index;
  BSTR getTypeName = SysAllocString(L"GetType");

  VariantInit(&typeNameParam);
  VariantInit(&getTypeRetVal);
  VariantInit(&nullObject);

  // Instantiate a dummy class just so we can get a System.Type instance
  hresult = CoCreateInstance(CLSID_Object, NULL, 
    CLSCTX_INPROC_SERVER, IID_IUnknown, (void **)&pUnk);
  if (FAILED(hresult)) goto cleanup;
  
  // Get the _Object interface so we can call GetType
  hresult = pUnk->QueryInterface(IID__Object, (void**)&pObj);
  if (FAILED(hresult)) goto cleanup;

  // Call _Object.GetType
  hresult = pObj->GetType(&pType);
  if (FAILED(hresult)) goto cleanup;

  // Call the instance Type.GetType method (inherited from Object)
  // in order to get the type for System.Type rather than the type for
  // System.Object
  hresult = pType->GetType(&pTypeOfType);
  if (FAILED(hresult)) goto cleanup;

  // Prepare a 1-element array containing the passed-in type name
  // to pass to the static Type.GetType method
  psa = SafeArrayCreateVector(VT_VARIANT, 0, 1);
  typeNameParam.vt = VT_BSTR;
  typeNameParam.bstrVal = typeName;
  index = 0;
  hresult = SafeArrayPutElement(psa, &index, &typeNameParam);
  if (FAILED(hresult)) goto cleanup;

  // Invoke the static Type.GetType method using reflection on the
  // type for System.Type in order to get the desired type
  nullObject.vt = VT_EMPTY;
  hresult = pTypeOfType->InvokeMember_3(getTypeName,
    (BindingFlags)(BindingFlags_InvokeMethod | BindingFlags_Public | 
    BindingFlags_Static | BindingFlags_FlattenHierarchy), NULL, 
    nullObject, psa, &getTypeRetVal);
  if (FAILED(hresult)) goto cleanup;

  // Get the _Type interface so we can call the static InvokeMember
  // method on the desired type to invoke the desired static member
  hresult = getTypeRetVal.punkVal->QueryInterface(IID__Type, 
    (void**)&pDesiredType);
  if (FAILED(hresult)) goto cleanup;

  // Invoke the desired static member
  pDesiredType->InvokeMember_3(memberName, (BindingFlags)(memberType |
    BindingFlags_Public | BindingFlags_Static | 
    BindingFlags_FlattenHierarchy), NULL, nullObject, parameters, 
    retVal);
  if (FAILED(hresult)) goto cleanup;

cleanup:
  if (pUnk) pUnk->Release();
  if (pObj) pObj->Release();
  if (pType) pType->Release();
  if (pTypeOfType) pTypeOfType->Release();
  if (pDesiredType) pDesiredType->Release();
  if (getTypeName) SysFreeString(getTypeName);
  SafeArrayDestroy(psa);
  VariantClear(&typeNameParam);
  VariantClear(&getTypeRetVal);
  VariantClear(&nullObject);

  return hresult;
};

int main(int argc, char* argv[])
{
  HRESULT hresult;
  VARIANT retVal;
  _AppDomain* pDomain = NULL;
  BSTR typeName1, typeName2, memberName1, memberName2, directory;

  // Initialize COM
  hresult = CoInitializeEx(NULL, COINIT_MULTITHREADED);
  if (FAILED(hresult))
  {
    printf("ERROR: Cannot initialize COM: 0x%x\n", hresult);
    return -1;
  }

  VariantInit(&retVal);

  // ------------------------------------------
  // Example 1: Calling System.Console.ReadLine
  // ------------------------------------------

  typeName1 = SysAllocString(L"System.Console");
  memberName1 = SysAllocString(L"ReadLine");

  printf("Type in something followed by Enter: ");

  hresult = InvokeStaticMember(typeName1, memberName1, 
    BindingFlags_InvokeMethod, NULL, &retVal);
  if (FAILED(hresult))
  {
    printf("ERROR: Invocation failed: 0x%x\n", hresult);
    return -1;
  }

  wprintf(L"You typed: '%s'\n", retVal.bstrVal);
  
  // -------------------------------------------------
  // Example 2: Calling System.AppDomain.CurrentDomain
  // -------------------------------------------------

  typeName2 = SysAllocString(L"System.AppDomain");
  memberName2 = SysAllocString(L"CurrentDomain");

  hresult = InvokeStaticMember(typeName2, memberName2, 
    BindingFlags_GetProperty, NULL, &retVal);
  if (FAILED(hresult))
  {
    printf("ERROR: Invocation failed: 0x%x\n", hresult);
    return -1;
  }

  // Get the _AppDomain interface from the returned IUnknown pointer
  hresult = retVal.punkVal->QueryInterface(IID__AppDomain, 
    (void**)&pDomain);
  if (FAILED(hresult)) 
  { 
    printf("ERROR: Could not get _AppDomain interface pointer: 0x%x\n",
      hresult);
    return -1;
  }

  // Call the BaseDirectory property on the _AppDomain instance
  pDomain->get_BaseDirectory(&directory);
  wprintf(L"Base directory of the current domain: '%s'\n", directory);

  CoUninitialize();
  return 0;
};