// To compile:
// csc /t:library ComFriendlyForm.cs /r:System.Windows.Forms.dll /r:System.Drawing.dll /r:System.dll

using System;
using System.Globalization;
using System.Windows.Forms;
using System.ComponentModel;
using System.Runtime.InteropServices;

// The source interface representing all events for Form
[InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
public interface IFormEvents
{
  // Component's event
  [DispId(1)] void Disposed (object sender, EventArgs e);

  // Control's events with COM-visible parameters
  [DispId(2)] void BackColorChanged (object sender, EventArgs e);
  [DispId(3)] void BackgroundImageChanged (object sender, EventArgs e);
  [DispId(4)] void BindingContextChanged (object sender, EventArgs e);
  [DispId(5)] void CausesValidationChanged (object sender, EventArgs e);
  [DispId(6)] void Click (object sender, EventArgs e);
  [DispId(7)] void ContextMenuChanged (object sender, EventArgs e);
  [DispId(8)] void CursorChanged (object sender, EventArgs e);
  [DispId(9)] void DockChanged (object sender, EventArgs e);
  [DispId(10)] void DoubleClick (object sender, EventArgs e);
  [DispId(11)] void DragDrop (object sender, DragEventArgs e);
  [DispId(12)] void DragEnter (object sender, DragEventArgs e);
  [DispId(13)] void DragLeave (object sender, EventArgs e);
  [DispId(14)] void DragOver (object sender, DragEventArgs e);
  [DispId(15)] void EnabledChanged (object sender, EventArgs e);
  [DispId(16)] void Enter (object sender, EventArgs e);
  [DispId(17)] void FontChanged (object sender, EventArgs e);
  [DispId(18)] void ForeColorChanged (object sender, EventArgs e);
  [DispId(19)] void GiveFeedback (object sender, GiveFeedbackEventArgs e);
  [DispId(20)] void GotFocus (object sender, EventArgs e);
  [DispId(21)] void HandleCreated (object sender, EventArgs e);
  [DispId(22)] void HandleDestroyed (object sender, EventArgs e);
  [DispId(23)] void HelpRequested (object sender, HelpEventArgs e);
  [DispId(24)] void ImeModeChanged (object sender, EventArgs e);
  [DispId(25)] void KeyDown (object sender, KeyEventArgs e);
  [DispId(26)] void KeyPress (object sender, KeyPressEventArgs e);
  [DispId(27)] void KeyUp (object sender, KeyEventArgs e);
  [DispId(28)] void Leave (object sender, EventArgs e);
  [DispId(29)] void LocationChanged (object sender, EventArgs e);
  [DispId(30)] void LostFocus (object sender, EventArgs e);
  [DispId(31)] void MouseDown (object sender, MouseEventArgs e);
  [DispId(32)] void MouseEnter (object sender, EventArgs e);
  [DispId(33)] void MouseHover (object sender, EventArgs e);
  [DispId(34)] void MouseLeave (object sender, EventArgs e);
  [DispId(35)] void MouseMove (object sender, MouseEventArgs e);
  [DispId(36)] void MouseUp (object sender, MouseEventArgs e);
  [DispId(37)] void MouseWheel (object sender, MouseEventArgs e);
  [DispId(38)] void Move (object sender, EventArgs e);
  [DispId(39)] void ParentChanged (object sender, EventArgs e);
  [DispId(40)] void Resize (object sender, EventArgs e);
  [DispId(41)] void RightToLeftChanged (object sender, EventArgs e);
  [DispId(42)] void SizeChanged (object sender, EventArgs e);
  [DispId(43)] void StyleChanged (object sender, EventArgs e);
  [DispId(44)] void SystemColorsChanged (object sender, EventArgs e);
  [DispId(45)] void TabIndexChanged (object sender, EventArgs e);
  [DispId(46)] void TabStopChanged (object sender, EventArgs e);
  [DispId(47)] void TextChanged (object sender, EventArgs e);
  [DispId(48)] void Validated (object sender, EventArgs e);
  [DispId(49)] void VisibleChanged (object sender, EventArgs e);
  [DispId(50)] void QueryAccessibilityHelp (object sender, 
    QueryAccessibilityHelpEventArgs e);
  [DispId(51)] void QueryContinueDrag (object sender, 
    QueryContinueDragEventArgs e);

  // Form's events with COM-visible parameters
  [DispId(52)] void Activated (object sender, EventArgs e);
  [DispId(53)] void Closed (object sender, EventArgs e);
  [DispId(54)] void Deactivate (object sender, EventArgs e);
  [DispId(55)] void Load (object sender, EventArgs e);
  [DispId(56)] void MaximizedBoundsChanged (object sender, EventArgs e);
  [DispId(57)] void MaximumSizeChanged (object sender, EventArgs e);
  [DispId(58)] void MdiChildActivate (object sender, EventArgs e);
  [DispId(59)] void MenuComplete (object sender, EventArgs e);
  [DispId(60)] void MenuStart (object sender, EventArgs e);
  [DispId(61)] void MinimumSizeChanged (object sender, EventArgs e);

  // Control's events with COM-invisible parameters
  [DispId(62)] void ChangeUICues (object sender, bool changeFocus,
    bool changeKeyboard, bool showFocus, bool showKeyboard);
  [DispId(63)] void ControlAdded (object sender, IComponent control);
  [DispId(64)] void ControlRemoved (object sender, IComponent control);
  [DispId(65)] void Invalidated (object sender, int x, int y, int width, 
    int height);
  [DispId(66)] void Layout (object sender, IComponent affectedControl,
    string affectedProperty);
  [DispId(67)] void Paint (object sender, IntPtr hdc, int x, int y, 
    int width, int height);
  [DispId(68)] void Validating (object sender, ref bool cancel);

  // Form's events with COM-invisible parameters
  [DispId(69)] void Closing (object sender, ref bool cancel);
  [DispId(70)] void InputLanguageChanged (object sender, byte charSet, 
    CultureInfo culture, IntPtr inputLanguageHandle,
    string inputLanguageLayoutName);
  [DispId(71)] void InputLanguageChanging (object sender, ref bool cancel,
    bool sysCharSet, CultureInfo culture, IntPtr inputLanguageHandle,
    string inputLanguageLayoutName);
}

// A Windows Form that exposes its events to COM using connection points
[ComSourceInterfaces(typeof(IFormEvents))]
public class ComFriendlyForm : Form
{
  //-------------------------------------------------------
  // Delegates for the COM-friendly event handlers
  //-------------------------------------------------------

  public delegate void UICuesExpandedEventHandler(object sender,
    bool changeFocus, bool changeKeyboard, bool showFocus,
    bool showKeyboard);

  public delegate void ControlExpandedEventHandler(object sender,
    IComponent control);

  public delegate void InvalidateExpandedEventHandler(object sender, 
    int x, int y, int width, int height);

  public delegate void LayoutExpandedEventHandler(object sender,
    IComponent affectedControl, string affectedProperty);

  public delegate void PaintExpandedEventHandler(object sender,
    IntPtr hdc, int x, int y, int width, int height);

  public delegate void CancelExpandedEventHandler(object sender,
    ref bool cancel);

  public delegate void InputLanguageChangedExpandedEventHandler(
    object sender, byte charSet, CultureInfo culture,
    IntPtr inputLanguageHandle, string inputLanguageLayoutName);

  public delegate void InputLanguageChangingExpandedEventHandler(
    object sender, ref bool cancel, bool sysCharSet, CultureInfo culture,
    IntPtr inputLanguageHandle, string inputLanguageLayoutName);

  //-------------------------------------------------------
  // New events using COM-friendly delegates 
  // (with the same names as the unfriendly events)
  //-------------------------------------------------------

  public new event UICuesExpandedEventHandler ChangeUICues;
  public new event ControlExpandedEventHandler ControlAdded;
  public new event ControlExpandedEventHandler ControlRemoved;
  public new event InvalidateExpandedEventHandler Invalidated;
  public new event LayoutExpandedEventHandler Layout;
  public new event PaintExpandedEventHandler Paint;
  public new event CancelExpandedEventHandler Validating;
  public new event CancelExpandedEventHandler Closing;
  public new event InputLanguageChangedExpandedEventHandler 
    InputLanguageChanged;
  public new event InputLanguageChangingExpandedEventHandler 
    InputLanguageChanging;

  //-------------------------------------------------------
  // Overridden On<Event> methods that not only call the
  // base On<Event> methods but also raise the new COM-friendly
  // events in case anyone's listening
  //-------------------------------------------------------

  protected override void OnChangeUICues(UICuesEventArgs e) 
  {
    base.OnChangeUICues(e);
    if (ChangeUICues != null) 
    {
      ChangeUICues(this, e.ChangeFocus, e.ChangeKeyboard, e.ShowFocus,
        e.ShowKeyboard);
    }
  }

  protected override void OnControlAdded(ControlEventArgs e) 
  {
    base.OnControlAdded(e);
    if (ControlAdded != null) 
      ControlAdded(this, e.Control);
  }

  protected override void OnControlRemoved(ControlEventArgs e) 
  {
    base.OnControlRemoved(e);
    if (ControlRemoved != null) 
      ControlRemoved(this, e.Control);
  }

  protected override void OnInvalidated(InvalidateEventArgs e) 
  {
    base.OnInvalidated(e);
    if (Invalidated != null) 
    {
      Invalidated(this, e.InvalidRect.X, e.InvalidRect.Y,
	    e.InvalidRect.Width, e.InvalidRect.Height);
    }
  }

  protected override void OnLayout(LayoutEventArgs e) 
  {
    base.OnLayout(e);
    if (Layout != null) 
      Layout(this, e.AffectedControl, e.AffectedProperty);
  }

  protected override void OnPaint(PaintEventArgs e) 
  {
    base.OnPaint(e);
    if (Paint != null) 
    {
      IntPtr hdc = e.Graphics.GetHdc();
      try
      {
        Paint(this, hdc, e.ClipRectangle.X,
          e.ClipRectangle.Y, e.ClipRectangle.Width, 
          e.ClipRectangle.Height);
      }
      finally { e.Graphics.ReleaseHdc(hdc); }
    }
  }
 
  protected override void OnInputLanguageChanged(
    InputLanguageChangedEventArgs e) 
  {
    base.OnInputLanguageChanged(e);
    if (InputLanguageChanged != null) 
    {
      InputLanguageChanged(this, e.CharSet, e.Culture,
        e.InputLanguage.Handle, e.InputLanguage.LayoutName);
    }
  }

  //-------------------------------------------------------
  // On<Event> methods that handle raised events then raise
  // the corresponding COM-friendly events, done this way
  // due to the by-reference parameter
  //-------------------------------------------------------

  private void HandleValidating(object sender, CancelEventArgs e) 
  {
    if (Validating != null) 
    {
      // Pass the CancelEventArgs.Cancel value by-reference
      bool cancel = e.Cancel;
      Validating(this, ref cancel);
      e.Cancel = cancel;
    }
  }

  private void HandleClosing(object sender, CancelEventArgs e) 
  {
    if (Closing != null) 
    {
      // Pass the CancelEventArgs.Cancel value by-reference
      bool cancel = e.Cancel;
      Closing(this, ref cancel);
      e.Cancel = cancel;
    }
  }

  private void HandleInputLanguageChanging(object sender, 
    InputLanguageChangingEventArgs e) 
  {
    if (InputLanguageChanging != null) 
    {
      // Pass the CancelEventArgs.Cancel value by-reference
      bool cancel = e.Cancel;
      InputLanguageChanging(this, ref cancel, e.SysCharSet, e.Culture,
        e.InputLanguage.Handle, e.InputLanguage.LayoutName);
      e.Cancel = cancel;
    }
  }

  //-------------------------------------------------------
  // Constructor
  //-------------------------------------------------------
  public ComFriendlyForm()
  {
    base.Validating += new CancelEventHandler(HandleValidating);
    base.Closing += new CancelEventHandler(HandleClosing);
    base.InputLanguageChanging += 
      new InputLanguageChangingEventHandler(HandleInputLanguageChanging);

    this.Text = ".NET Windows Form";
  }
}