// To compile:
// csc /t:library Listing13_1cs.cs

using System.Collections;
using System.Runtime.InteropServices;

// Interface with "event handlers" for COM objects to implement
public interface IPhoneEvents
{
  void Ring();
  void CallerId(string callerName, ref byte [] callerPhoneNumber);
}

// Interface providing COM access to usable add/remove methods
public interface IPhoneEventHookup
{
  int Add(IPhoneEvents comObject);
  void Remove(int cookie);
}

// Delegates for the events
public delegate void RingEventHandler();
public delegate void CallerIdEventHandler(string callerName,
  byte [] callerPhoneNumber);

// The class with the events
[ClassInterface(ClassInterfaceType.None)]
public class Phone : IPhoneEventHookup
{
  // The two events
  public event RingEventHandler Ring;
  public event CallerIdEventHandler CallerId;

  // Collection of COM objects to call when event is raised
  Hashtable comObjects;
  // Number to uniquely identify COM sinks
  int cookie;

  // Constructor
  public Phone()
  {
    comObjects = new Hashtable();
    cookie = 0;

    // Hook up the class's private methods to the events
    Ring += new RingEventHandler(OnRing);
    CallerId += new CallerIdEventHandler(OnCallerId);
  }

  // Implementation of IPhoneEventHookup.Add
  int IPhoneEventHookup.Add(IPhoneEvents comObject)
  {
    cookie++;
    // Add object to the collection
    comObjects.Add(cookie, comObject);
    return cookie;
  }

  // Implementation of IPhoneEventHookup.Remove
  void IPhoneEventHookup.Remove(int cookie)
  {
    // Remove the object from the collection
    comObjects.Remove(cookie);
  }

  // Handles the Ring event and calls all COM objects that
  // added themselves to the list using IPhoneEventHookup.Add
  private void OnRing()
  {
    foreach (IPhoneEvents comObject in comObjects)
      comObject.Ring();
  }

  // Handles the CallerId event and calls all COM objects that
  // added themselves to the list using IPhoneEventHookup.Add
  private void OnCallerId(string callerName, byte [] callerPhoneNumber)
  {
    foreach (IPhoneEvents comObject in comObjects)
      comObject.CallerId(callerName, ref callerPhoneNumber);    
  }

  // More methods to communicate with hardware and raise the events...
}