' To compile:
' vbc /t:library Listing13_1vb.vb

Imports System.Collections
Imports System.Runtime.InteropServices

' Interface with "event handlers" for COM objects to implement
Public Interface IPhoneEvents
  Sub Ring()
  Sub CallerId(callerName As String, ByRef callerPhoneNumber() As Byte)
End Interface

' Interface providing COM access to usable add/remove methods
Public Interface IPhoneEventHookup
  Function Add(comObject As IPhoneEvents) As Integer
  Sub Remove(cookie As Integer)
End Interface

' Delegates for the events
Public Delegate Sub RingEventHandler()
Public Delegate Sub CallerIdEventHandler(callerName As String, callerPhoneNumber() As Byte)

' The class with the events
<ClassInterface(ClassInterfaceType.None)> _
Public Class Phone
  Implements IPhoneEventHookup

  ' The two events
  Public Event Ring As RingEventHandler
  Public Event CallerId As CallerIdEventHandler
  
  ' Collection of COM objects to call when event is raised
  Private comObjects As Hashtable
  ' Number to uniquely identify COM sinks
  Private cookie As Integer
 
  ' Constructor
  Public Sub New()
    comObjects = New Hashtable()
    cookie = 0

    ' Hook up the class's private methods to the events
    AddHandler Ring, AddressOf OnRing
    AddHandler CallerId, AddressOf OnCallerId
  End Sub

  ' Implementation of IPhoneEventHookup.Add
  Function Add(comObject As IPhoneEvents) As Integer Implements IPhoneEventHookup.Add
    cookie += 1
    ' Add object to the collection
    comObjects.Add(cookie, comObject)
    Return cookie
  End Function

  ' Implementation of IPhoneEventHookup.Remove
  Sub Remove(cookie As Integer) Implements IPhoneEventHookup.Remove
    ' Remove the object from the collection
    comObjects.Remove(cookie)
  End Sub

  ' Handles the Ring event and calls all COM objects that
  ' added themselves to the list using IPhoneEventHookup.Add
  Private Sub OnRing()
    Dim comObject As IPhoneEvents
    For Each comObject In comObjects
      comObject.Ring()
    Next
  End Sub
 
  ' Handles the CallerId event and calls all COM objects that
  ' added themselves to the list using IPhoneEventHookup.Add
  Private Sub OnCallerId(callerName As String, callerPhoneNumber() As Byte)
    Dim comObject As IPhoneEvents
      For Each comObject In comObjects
        comObject.CallerId(callerName, callerPhoneNumber)
      Next
  End Sub

  ' More methods to communicate with hardware and raise the events...

End Class