' To compile:
' vbc /t:library CustomDispatch.vb

Imports System
Imports System.Reflection
Imports System.Globalization
Imports System.Collections
Imports System.Runtime.InteropServices

Public Class CustomDispatch
  Implements IReflect

  Private t As Type
  Private names As Hashtable
   
  ' Well-known HRESULT returned by IDispatch.Invoke:
  Private Const DISP_E_MEMBERNOTFOUND As Integer = &H80020003

  ' Constructor
  Public Sub New()
    t = Me.GetType()
      
    ' Fill Hashtable with explicit DISPIDs and their member names
    names = New Hashtable()
    Dim ids() As DispIdAttribute
    Dim mi As MemberInfo

    For Each mi In t.GetMembers()
      ids = CType(mi.GetCustomAttributes(GetType(DispIdAttribute), False), DispIdAttribute())
         
      If ids.Length > 0 Then
        names.Add(ids(0).Value, mi.Name)
      End If
    Next mi
  End Sub
    
  ' Called by CLR to get DISPIDs and names for properties
  Function GetProperties(bindingAttr As BindingFlags) As PropertyInfo() Implements IReflect.GetProperties
    GetProperties = t.GetProperties(bindingAttr)
  End Function
   
  ' Called by CLR to get DISPIDs and names for fields
  Function GetFields(bindingAttr As BindingFlags) As FieldInfo() Implements IReflect.GetFields
    GetFields = t.GetFields(bindingAttr)
  End Function
   
  ' Called by CLR to get DISPIDs and names for methods
  Function GetMethods(bindingAttr As BindingFlags) As MethodInfo() Implements IReflect.GetMethods
    GetMethods = t.GetMethods(bindingAttr)
  End Function
   
  ' Called by CLR to invoke a member
  Function InvokeMember(name As String, invokeAttr As BindingFlags, binder As Binder, _
    target As Object, args() As Object, modifiers() As ParameterModifier, culture As CultureInfo, _
    namedParameters() As String) As Object Implements IReflect.InvokeMember

    ' Special case for any member marked with DispIdAttribute
    If name.StartsWith("[DISPID=") Then
      Dim dispid As Integer = Convert.ToInt32(name.Substring(8, name.Length-9), 10)
      name = CStr(names(dispid))
    ' Special case for DISPID_EVALUATE (-5)
    ElseIf name.Equals("[DISPID=-5]") Then
      name = "Evaluate"
    End If
      
    Try
      InvokeMember = t.InvokeMember(name, invokeAttr, binder, target, args, modifiers, culture, namedParameters)
    Catch ex As MissingMemberException
      Throw New COMException(ex.Message, DISP_E_MEMBERNOTFOUND)
    End Try
  End Function
   
  Function GetField(name As String, bindingAttr As BindingFlags) As FieldInfo Implements IReflect.GetField
    GetField = t.GetField(name, bindingAttr)
  End Function
   
  Function GetMember(name As String, bindingAttr As BindingFlags) As MemberInfo() Implements IReflect.GetMember
    GetMember = t.GetMember(name, bindingAttr)
  End Function
   
  Function GetMembers(bindingAttr As BindingFlags) As MemberInfo() Implements IReflect.GetMembers
    GetMembers = t.GetMembers(bindingAttr)
  End Function
   
  Overloads Function GetMethod(name As String, bindingAttr As BindingFlags) As MethodInfo Implements IReflect.GetMethod
    GetMethod = t.GetMethod(name, bindingAttr)
  End Function
   
  Overloads Function GetMethod(name As String, bindingAttr As BindingFlags, binder As Binder, _
    types() As Type, modifiers() As ParameterModifier) As MethodInfo Implements IReflect.GetMethod

    GetMethod = t.GetMethod(name, bindingAttr, binder, types, modifiers)
  End Function
   
  Overloads Function GetProperty(name As String, bindingAttr As BindingFlags, binder As Binder, _
    returnType As Type, types() As Type, modifiers() As ParameterModifier) As PropertyInfo Implements IReflect.GetProperty

    GetProperty = t.GetProperty(name, bindingAttr, binder, returnType, types, modifiers)
  End Function
   
  Overloads Function GetProperty(name As String, bindingAttr As BindingFlags) As PropertyInfo Implements IReflect.GetProperty
    GetProperty = t.GetProperty(name, bindingAttr)
  End Function
  
  ReadOnly Property UnderlyingSystemType() As Type Implements IReflect.UnderlyingSystemType
    Get
      Return t.UnderlyingSystemType
    End Get
  End Property

End Class