// FileWriter.cpp : Implementation of CFileWriter
#include "stdafx.h"
#include "FileComponent.h"
#include "FileWriter.h"
#include "d:\clr\src\inc\CorError.h" // Contains definitions of .NET HRESULTs
									 // You may need to modify your settings to find the header file.

/////////////////////////////////////////////////////////////////////////////
// CFileWriter

//
// ISupportErrorInfo.InterfaceSupportsErrorInfo
//
STDMETHODIMP CFileWriter::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IFileWriter,
		&IID_IDisposable
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

//
// Constructor
//
CFileWriter::CFileWriter()
{
	filePointer = fopen("tempfile.txt", "a+");
}

//
// Destructor
//
CFileWriter::~CFileWriter()
{
	if (NULL != filePointer)
      fclose(filePointer);
}

//
// WriteLine
//
STDMETHODIMP CFileWriter::WriteLine(BSTR message)
{
	if (NULL == filePointer)
	{
		// We can't throw an ObjectDisposedException from COM since
		// it shares an HRESULT with its base InvalidOperationException
		// class, but this is close.  At least we can give the exception
		// the message for ObjectDisposedException!

		ICreateErrorInfo *pcerrinfo;
		IErrorInfo *perrinfo;
		HRESULT hr;
 
		BSTR description = SysAllocString(L"Cannot access the disposed FileWriter object.");
        BSTR source = SysAllocString(L"File Components");
   
		hr = CreateErrorInfo(&pcerrinfo);
        if (SUCCEEDED(hr))
		{
          pcerrinfo->SetDescription(description);
          pcerrinfo->SetSource(source);

		  hr = pcerrinfo->QueryInterface(IID_IErrorInfo, (void**)&perrinfo);
		  if (SUCCEEDED(hr))
		  {
				SetErrorInfo(0, perrinfo);
				perrinfo->Release();
		  }

          pcerrinfo->Release();
		}

		return COR_E_INVALIDOPERATION;
	}
	else
	{
		fwprintf(filePointer, message);
		fwprintf(filePointer, L"\n");
		return S_OK;
	}
}

//
// IDisposable.Dispose
//
STDMETHODIMP CFileWriter::Dispose()
{
	// IDisposable.Dispose is idempotent
	if (NULL != filePointer)
	{
      fclose(filePointer);
	  filePointer = NULL;
	}
	return S_OK;
}