' To compile:
' vbc Listing19_10vb.vb ErrorHelper.vb

Imports System
Imports System.Text
Imports System.Collections
Imports System.Runtime.InteropServices

Public Class RegistryKey
  Private hKey As IntPtr
   
  Private Declare Auto Function RegOpenKey Lib "advapi32.dll" _
    (hKey As IntPtr, lpSubKey As String, ByRef phkResult As IntPtr) As Integer
   
  Private Declare Function RegCloseKey Lib "advapi32.dll" _
    (hKey As IntPtr) As Integer
   
  ' Constructor
  Public Sub New(rootKey As IntPtr, subKey As String)
    ' Open the key
    Dim result As Integer = RegOpenKey(rootKey, subKey, hKey)
    
    If result <> 0 Then
      Throw New ApplicationException("Could not open registry key: " + ErrorHelper.GetErrorMessage(result))
    End If
  End Sub
   
  ' Read-only property
  Public ReadOnly Property Handle() As IntPtr
    Get
      Return hKey
    End Get
  End Property
    
  ' Finalizer
  Protected Overloads Overrides Sub Finalize()
    ' Close the key
    RegCloseKey(hKey)
    MyBase.Finalize()
  End Sub
End Class

Public Class RegistryKeyEnumerator
  Implements IEnumerator

  Private ERROR_NO_MORE_ITEMS As Integer = 259
 
  Private hKey As IntPtr
  Private currentSubKeyIndex As Integer
  Private currentSubKeyText As StringBuilder
   
  Private Declare Auto Function RegEnumKey Lib "advapi32.dll" _
    (hKey As IntPtr, dwIndex As Integer, <Out> lpName As StringBuilder, cchName As Integer) As Integer
   
  ' Create a new enumerator based on the passed-in information
  Public Sub New(hKeyRoot As IntPtr, subKey As String)
    Dim key As New RegistryKey(hKeyRoot, subKey)
    hKey = key.Handle
    currentSubKeyText = New StringBuilder(256)
    Reset()
  End Sub
   
  Public Sub Reset() Implements IEnumerator.Reset
    currentSubKeyIndex = - 1
  End Sub
  
  Public Function MoveNext() As Boolean Implements IEnumerator.MoveNext
    currentSubKeyIndex += 1
    Dim result As Integer = RegEnumKey(hKey, currentSubKeyIndex, currentSubKeyText, currentSubKeyText.Capacity)

    If result = 0 Then
      Return True
    ElseIf result = ERROR_NO_MORE_ITEMS Then
      Return False
    Else
      Throw New ApplicationException(ErrorHelper.GetErrorMessage(result))
    End If
  End Function
 
  Public ReadOnly Property Current() As Object Implements IEnumerator.Current
    Get
      Return currentSubKeyText.ToString()
    End Get
  End Property
End Class

Public Class Client
  Private Shared ReadOnly HKEY_LOCAL_MACHINE As New IntPtr(&H80000002)
   
  Public Shared Sub Main()
    Dim enumerator As New RegistryKeyEnumerator(HKEY_LOCAL_MACHINE, "SOFTWARE")
      
    ' Force a garbage collection for demonstration
    GC.Collect()
    GC.WaitForPendingFinalizers()
      
    enumerator.Reset()

    ' This causes an ApplicationException:
    While enumerator.MoveNext()
      Console.WriteLine(enumerator.Current)
    End While
  End Sub
End Class