// To compile:
// csc Listing19_11cs.cs ErrorHelper.cs

using System;
using System.Text;
using System.Collections;
using System.Runtime.InteropServices;

public class RegistryKey : IDisposable
{
  IntPtr hKey;

  [DllImport("advapi32.dll", CharSet=CharSet.Auto)]
  static extern int RegOpenKey(IntPtr hKey, string lpSubKey,
    out IntPtr phkResult);

  [DllImport("advapi32.dll")]
  static extern int RegCloseKey(IntPtr hKey);

  // Constructor
  public RegistryKey(IntPtr rootKey, string subKey)
  {
    // Open the key
    int result = RegOpenKey(rootKey, subKey, out hKey);
    if (result != 0)
      throw new ApplicationException("Could not open registry key: " +
        ErrorHelper.GetErrorMessage(result));
  }
  
  // Read-only property
  public IntPtr Handle
  {
    get { return hKey; }
  }

  // Finalizer
  ~RegistryKey()
  {
    // Close the key
    RegCloseKey(hKey);
  }

  public void Dispose()
  {
    // Close the key
    RegCloseKey(hKey);
    GC.SuppressFinalize(this);
  }
}

public class RegistryKeyEnumerator : IEnumerator, IDisposable
{
  const int ERROR_NO_MORE_ITEMS = 259;

  IntPtr hKey;
  int currentSubKeyIndex;
  StringBuilder currentSubKeyText;
  RegistryKey key;

  [DllImport("advapi32.dll", CharSet=CharSet.Auto)]
  static extern int RegEnumKey(IntPtr hKey, int dwIndex,
    [Out] StringBuilder lpName, int cchName);
 
  // Finalizer
  ~RegistryKeyEnumerator()
  {
    if (key != null) key.Dispose();
  }

  public void Dispose()
  {
    if (key != null) key.Dispose();
    GC.SuppressFinalize(this);
  }

  // Create a new enumerator based on the passed-in information
  public RegistryKeyEnumerator(IntPtr hKeyRoot, string subKey)
  {
    key = new RegistryKey(hKeyRoot, subKey);
    hKey = key.Handle;
    currentSubKeyText = new StringBuilder(256);
    Reset();
  }

  public void Reset()
  {
    currentSubKeyIndex = -1;
  }

  public bool MoveNext()
  {
    currentSubKeyIndex++;
    int result = RegEnumKey(hKey, currentSubKeyIndex, currentSubKeyText,
      currentSubKeyText.Capacity);

    if (result == 0)
      return true;
    else if (result == ERROR_NO_MORE_ITEMS)
      return false;
    else
      throw new ApplicationException(ErrorHelper.GetErrorMessage(result));
  }

  public object Current
  {
    get { return currentSubKeyText.ToString(); }
  }
}

public class Client
{
  static readonly IntPtr HKEY_LOCAL_MACHINE = 
    new IntPtr(unchecked((int)0x80000002));

  public static void Main()
  {
    using (RegistryKeyEnumerator enumerator = 
             new RegistryKeyEnumerator(HKEY_LOCAL_MACHINE, "SOFTWARE"))
    {
      // Force a garbage collection for demonstration
      GC.Collect();
      GC.WaitForPendingFinalizers();

      enumerator.Reset();
      while (enumerator.MoveNext())
        Console.WriteLine(enumerator.Current);
    }
  }
}