// To compile:
// csc Listing19_13cs.cs

using System;
using System.Runtime.InteropServices;

internal struct COORD 
{
  internal short X;
  internal short Y;
}

[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Unicode)]
internal struct KEY_EVENT_RECORD
{
  internal bool bKeyDown;
  internal ushort wRepeatCount;
  internal ushort wVirtualKeyCode;
  internal ushort wVirtualScanCode;
  internal char UnicodeChar;
  internal uint dwControlKeyState;
}

internal struct MOUSE_EVENT_RECORD 
{
  internal COORD dwMousePosition;
  internal uint dwButtonState;
  internal uint dwControlKeyState;
  internal uint dwEventFlags;
}

internal struct WINDOW_BUFFER_SIZE_RECORD 
{
  internal COORD dwSize;
}

internal struct MENU_EVENT_RECORD 
{
  internal uint dwCommandId;
}

internal struct FOCUS_EVENT_RECORD 
{
  internal bool bSetFocus;
}

internal struct INPUT_RECORD
{
  internal EventTypes EventType;
  internal KEY_EVENT_RECORD KeyEvent; 
  internal MOUSE_EVENT_RECORD MouseEvent; 
  internal WINDOW_BUFFER_SIZE_RECORD WindowBufferSizeEvent; 
  internal MENU_EVENT_RECORD MenuEvent; 
  internal FOCUS_EVENT_RECORD FocusEvent; 
}

internal enum EventTypes : ushort
{
  KEY_EVENT                = 0x01, 
  MOUSE_EVENT              = 0x02,
  WINDOW_BUFFER_SIZE_EVENT = 0x04, 
  MENU_EVENT               = 0x08, 
  FOCUS_EVENT              = 0x10 
}

// Constants used with PInvoke methods
internal class Constants
{
  // Standard input, output, and error
  internal const int STD_INPUT_HANDLE  = -10;
  internal const int STD_OUTPUT_HANDLE = -11;
  internal const int STD_ERROR_HANDLE  = -12;

  // Returned by GetStdHandle when an error occurs
  internal static readonly IntPtr INVALID_HANDLE_VALUE = new IntPtr(-1);
}

public class Echo
{
  [DllImport("kernel32.dll", SetLastError=true)]
  private static extern IntPtr GetStdHandle(int nStdHandle);

  [DllImport("kernel32.dll", SetLastError=true)]
  static extern bool ReadConsoleInput(IntPtr hConsoleInput, 
    [Out] INPUT_RECORD [] lpBuffer, uint nLength, 
    out uint lpNumberOfEventsRead);

  public static void Main()
  {
    IntPtr stdin = GetStdHandle(Constants.STD_INPUT_HANDLE);

    if (stdin == Constants.INVALID_HANDLE_VALUE)
      throw new Exception("Unable to get standard input handle");

    INPUT_RECORD [] records = new INPUT_RECORD[1];
    uint eventsRead = 0;

    // Loop forever
    while (true)
    {
      if (ReadConsoleInput(stdin, records, (uint)records.Length,
        out eventsRead))
      {
        if (records[0].EventType == EventTypes.KEY_EVENT &&
          records[0].KeyEvent.bKeyDown)
        {
          Console.WriteLine("You pressed '" + 
            records[0].KeyEvent.UnicodeChar + "' (" +
            records[0].KeyEvent.wVirtualKeyCode + ")!");
        }
      }
    }
  }
}