// To compile:
// csc Listing19_14cs.cs

using System;
using System.Runtime.InteropServices;

internal struct CONSOLE_SCREEN_BUFFER_INFO 
{
  internal COORD dwSize;
  internal COORD dwCursorPosition;
  internal ushort wAttributes;
  internal SMALL_RECT srWindow;
  internal COORD dwMaximumWindowSize;
}

internal struct COORD 
{
  internal short X;
  internal short Y;
}

internal struct SMALL_RECT 
{
  internal short Left;
  internal short Top;
  internal short Right;
  internal short Bottom;
}

// Constants used with PInvoke methods
internal class Constants
{
  // Standard input, output, and error
  internal const int STD_INPUT_HANDLE  = -10;
  internal const int STD_OUTPUT_HANDLE = -11;
  internal const int STD_ERROR_HANDLE  = -12;

  // Returned by GetStdHandle when an error occurs
  internal static readonly IntPtr INVALID_HANDLE_VALUE = new IntPtr(-1);
}

public class Cls
{
  [DllImport("kernel32.dll", SetLastError=true)]
  static extern IntPtr GetStdHandle(int nStdHandle);

  [DllImport("kernel32.dll", SetLastError=true)]
  static extern bool GetConsoleScreenBufferInfo(IntPtr hConsoleOutput,
    out CONSOLE_SCREEN_BUFFER_INFO lpConsoleScreenBufferInfo);

  [DllImport("kernel32.dll", SetLastError=true)]
  static extern bool FillConsoleOutputCharacter(IntPtr hConsoleOutput, 
    char cCharacter, uint nLength, COORD dwWriteCoord, out uint 
    lpNumberOfCharsWritten);

  [DllImport("kernel32.dll", SetLastError=true)]
  static extern bool FillConsoleOutputAttribute(IntPtr hConsoleOutput, 
    ushort wAttribute, uint nLength, COORD dwWriteCoord, out uint 
    lpNumberOfAttrsWritten);

  [DllImport("kernel32.dll", SetLastError=true)]
  static extern bool SetConsoleCursorPosition(IntPtr hConsoleOutput, 
    COORD dwCursorPosition);

  public static void Main()
  {
    IntPtr stdout = GetStdHandle(Constants.STD_OUTPUT_HANDLE);
    uint length;

    if (stdout == Constants.INVALID_HANDLE_VALUE)
      throw new Exception("Unable to get standard output handle");

    // Place the cursor at the beginning of the window (0,0)
    COORD coord = new COORD();

    // Get number of character cells in the buffer
    CONSOLE_SCREEN_BUFFER_INFO csbi = new CONSOLE_SCREEN_BUFFER_INFO();
    if (!GetConsoleScreenBufferInfo(stdout, out csbi))
      throw new Exception("Unable to get existing console settings");
    
    uint size = (uint)(csbi.dwSize.X * csbi.dwSize.Y);

    // Fill the entire screen with spaces
    if (!FillConsoleOutputCharacter(stdout, ' ', size, coord, out length))
      throw new Exception("Unable to fill console with spaces");

    // Set the buffer's attributes
    if (!FillConsoleOutputAttribute(stdout, csbi.wAttributes, size,
      coord, out length))
      throw new Exception("Unable to set console attributes");

    // Set the cursor's position to (0,0)
    if (!SetConsoleCursorPosition(stdout, coord))
      throw new Exception("Unable to set cursor position");
  }
}