' To compile:
' vbc Listing19_14vb.vb

Imports System
Imports System.Runtime.InteropServices

Structure CONSOLE_SCREEN_BUFFER_INFO
  Friend dwSize As COORD
  Friend dwCursorPosition As COORD
  Friend wAttributes As Short
  Friend srWindow As SMALL_RECT
  Friend dwMaximumWindowSize As COORD
End Structure

Structure COORD
  Friend X As Short
  Friend Y As Short
End Structure

Structure SMALL_RECT
  Friend Left As Short
  Friend Top As Short
  Friend Right As Short
  Friend Bottom As Short
End Structure

' Constants used with PInvoke methods
Class Constants
  ' Standard input, output, and error
  Friend Const STD_INPUT_HANDLE As Integer = -10
  Friend Const STD_OUTPUT_HANDLE As Integer = -11
  Friend Const STD_ERROR_HANDLE As Integer = -12
      
  ' Returned by GetStdHandle when an error occurs
  Friend Shared ReadOnly INVALID_HANDLE_VALUE As New IntPtr(-1)
End Class

Public Class Cls
 
  Private Declare Function GetStdHandle Lib "kernel32.dll" (nStdHandle As Integer) As IntPtr

  Private Declare Function GetConsoleScreenBufferInfo Lib "kernel32.dll" _
    (hConsoleOutput As IntPtr, <Out> ByRef lpConsoleScreenBufferInfo As CONSOLE_SCREEN_BUFFER_INFO) As Boolean

  Private Declare Ansi Function FillConsoleOutputCharacter Lib "kernel32.dll" Alias "FillConsoleOutputCharacterA" _
    (hConsoleOutput As IntPtr, cCharacter As Char, nLength As Integer, _
    dwWriteCoord As COORD, <Out> ByRef lpNumberOfCharsWritten As Integer) As Boolean

  Private Declare Function FillConsoleOutputAttribute Lib "kernel32.dll" _
    (hConsoleOutput As IntPtr, wAttribute As Short, nLength As Integer, dwWriteCoord As COORD, _
    <Out> ByRef lpNumberOfAttrsWritten As Integer) As Boolean

  Private Declare Function SetConsoleCursorPosition Lib "kernel32.dll" _
    (hConsoleOutput As IntPtr, dwCursorPosition As COORD) As Boolean
   
  Public Shared Sub Main()
    Dim stdout As IntPtr = GetStdHandle(Constants.STD_OUTPUT_HANDLE)
    Dim length As Integer
 
    If stdout.Equals(Constants.INVALID_HANDLE_VALUE) Then
      Throw New Exception("Unable to get standard output handle")
    End If 

    ' Place the cursor at the beginning of the window (0,0)
    Dim coord As New COORD()
      
    ' Get number of character cells in the buffer
    Dim csbi As New CONSOLE_SCREEN_BUFFER_INFO()
    If Not GetConsoleScreenBufferInfo(stdout, csbi) Then
      Throw New Exception("Unable to get existing console settings")
    End If 

    Dim size As Integer = CInt(csbi.dwSize.X) * CInt(csbi.dwSize.Y)

    ' Fill the entire screen with spaces
    If Not FillConsoleOutputCharacter(stdout, " "c, size, coord, length) Then
      Throw New Exception("Unable to fill console with spaces")
    End If 

    ' Set the buffer's attributes
    If Not FillConsoleOutputAttribute(stdout, csbi.wAttributes, size, coord, length) Then
      Throw New Exception("Unable to set console attributes")
    End If 

    ' Set the cursor's position to (0,0)
    If Not SetConsoleCursorPosition(stdout, coord) Then
      Throw New Exception("Unable to set cursor position")
    End If
  End Sub

End Class