// To compile:
// csc /t:library Listing19_15cs.cs 

using System;
using System.Runtime.InteropServices;

internal struct MULTI_QI
{
  internal IntPtr pIID;
  [MarshalAs(UnmanagedType.IUnknown)] internal object pItf;
  internal int hr;
}

[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Unicode)]
internal struct COSERVERINFO
{
  internal uint dwReserved1;
  internal string pwszName;
  internal IntPtr pAuthInfo;
  internal uint dwReserved2;
}

public class ComHelpers
{
  const uint CLSCTX_REMOTE_SERVER = 0x10;
  static readonly Guid IID_IUnknown = 
    new Guid("00000000-0000-0000-C000-000000000046");

  [DllImport("ole32.dll", CharSet=CharSet.Unicode, PreserveSig=false)]
  static extern void CLSIDFromProgIDEx(string lpszProgID, out Guid pclsid);

  [DllImport("ole32.dll", PreserveSig=false)]
  static extern void CoCreateInstanceEx(
    [MarshalAs(UnmanagedType.LPStruct)] Guid rclsid,
    [MarshalAs(UnmanagedType.IUnknown)] object punkOuter,
    uint dwClsCtx,
    [In] ref COSERVERINFO pServerInfo,
    uint cmq,
    [In, Out] MULTI_QI [] pResults);

  public static object CreateRemoteInstance(
    string progId, string serverName)
  {
    // Specify just the server name in the COSERVERINFO structure
    COSERVERINFO csi = new COSERVERINFO();
    csi.pwszName = serverName;
    csi.pAuthInfo = IntPtr.Zero;
    csi.dwReserved1 = 0;
    csi.dwReserved2 = 0;

    // Find the CLSID for the input ProgID
    Guid clsid;
    CLSIDFromProgIDEx(progId, out clsid);

    // Query for only one interface (IUnknown)
    MULTI_QI [] mqi = new MULTI_QI[1];
    mqi[0] = new MULTI_QI();

    GCHandle handle = new GCHandle();
    try
    {
      // Pin the Guid representing IID_IUnknown
      handle = GCHandle.Alloc(IID_IUnknown, GCHandleType.Pinned);

      // Set the field to the address of the Guid instance
      mqi[0].pIID = handle.AddrOfPinnedObject();
  
      // Make the PInvoke call
      CoCreateInstanceEx(clsid, null, 
        CLSCTX_REMOTE_SERVER, ref csi, 1, mqi);
    }
    finally
    {
      if (handle.IsAllocated) handle.Free();
    }

    // Return the RCW
    return mqi[0].pItf;
  }
}