// To compile:
// csc Listing19_1cs.cs ErrorHelper.cs

using System;
using System.Runtime.InteropServices;

public class CopyFile
{
  const uint CALLBACK_CHUNK_FINISHED = 0x0;
  const uint CALLBACK_STREAM_SWITCH  = 0x1;
  const uint PROGRESS_CONTINUE       = 0x0;

  delegate uint CopyProgressDelegate(long TotalFileSize, 
    long TotalBytesTransferred, long StreamSize, 
    long StreamBytesTransferred, uint dwStreamNumber, 
    uint dwCallbackReason, IntPtr hSourceFile,
    IntPtr hDestinationFile, IntPtr lpData);

  [DllImport("kernel32.dll", CharSet=CharSet.Auto, SetLastError=true)]
  static extern bool CopyFileEx(string lpExistingFileName, 
    string lpNewFileName, CopyProgressDelegate lpProgressRoutine,
    IntPtr lpData, [In] ref bool pbCancel, uint dwCopyFlags);

  static CopyProgressDelegate d;

  public static void Main()
  {
    bool cancel = false;
    d = new CopyProgressDelegate(OnProgress);

    Console.WriteLine("Copy file...");
    bool result;
    result = CopyFileEx("c:\\file1.txt", "c:\\file2.txt",
      d, IntPtr.Zero, ref cancel, 0);

    if (!result)
    {
      int errorCode = Marshal.GetLastWin32Error();
      string message = ErrorHelper.GetErrorMessage(errorCode);

      if (message == "")
        Console.WriteLine("Failed with error code " + errorCode);
      else
        Console.WriteLine("Failed: " + message);
    }
    else
    {
      Console.WriteLine("...Finished");
    }
  }

  // Callback method
  static uint OnProgress(long TotalFileSize, long TotalBytesTransferred,
    long StreamSize, long StreamBytesTransferred, uint dwStreamNumber, 
    uint dwCallbackReason, IntPtr hSourceFile, IntPtr hDestinationFile, 
    IntPtr lpData)
  {
    if (dwCallbackReason == CALLBACK_CHUNK_FINISHED)
    {
      Console.WriteLine("************* Chunk Finished *************");
      Console.WriteLine("File Bytes:   " + TotalBytesTransferred + 
        " of " + TotalFileSize);
      Console.WriteLine("Stream Bytes: " + StreamBytesTransferred + 
        " of " + StreamSize);
    }
    else if (dwCallbackReason == CALLBACK_STREAM_SWITCH)
    {
      Console.WriteLine("*************** New Stream ***************");
    }
    else
    {
      Console.WriteLine("************ Unknown Callback ************");
    }
    return PROGRESS_CONTINUE;
  }
}