// To compile:
// csc Listing19_2cs.cs

using System;
using System.Runtime.InteropServices;

public enum ControlSignals
{
  CTRL_C_EVENT        = 0,
  CTRL_BREAK_EVENT    = 1,
  CTRL_CLOSE_EVENT    = 2,
  CTRL_LOGOFF_EVENT   = 5,
  CTRL_SHUTDOWN_EVENT = 6
}

class HandlesControlSignals
{
  [DllImport("kernel32.dll")]
  static extern bool SetConsoleCtrlHandler(ConsoleCtrlDelegate HandlerRoutine, bool Add);

  delegate bool ConsoleCtrlDelegate(ControlSignals dwControlType);

  static ConsoleCtrlDelegate d;

  public static void Main()
  {
    // Required to prevent early collection of delegate
    d = new ConsoleCtrlDelegate(OnSignal);

    // Add our OnSignal method to the list of handlers
    SetConsoleCtrlHandler(d, true);

    Console.WriteLine("Press 'Q' (followed by Enter) to quit.");
    while (Console.ReadLine() != "Q") {}

    // Restore normal processing
    SetConsoleCtrlHandler(null, false);
  }

  public static bool OnSignal(ControlSignals dwControlType)
  {
    switch (dwControlType)
    {
      case ControlSignals.CTRL_C_EVENT:
        Console.WriteLine("[Ctrl+C]  Press 'Q' to quit.");
        return true;
      case ControlSignals.CTRL_BREAK_EVENT:
        Console.WriteLine("[Ctrl+Break]  Press 'Q' to quit.");
        return true;
      case ControlSignals.CTRL_CLOSE_EVENT:
        Console.WriteLine("[Window Close]  Press 'Q' to quit.");
        return true;
      default:
        return false;
    }
  }
}