// To compile:
// csc Listing19_3cs.cs

using System;
using System.Reflection;
using System.Reflection.Emit;
using System.Runtime.InteropServices;

public class InvokeFunctionPointer
{
  [DllImport("kernel32.dll")]
  static extern IntPtr LoadLibrary(string lpFileName);

  [DllImport("kernel32.dll")]
  static extern IntPtr GetProcAddress(IntPtr hModule, string lpProcName);

  public static void Main()
  {
    // Call any unmanaged method that returns a function pointer.
    // GetProcAddress is a canonical example.
    IntPtr hModule = LoadLibrary("user32.dll");
    IntPtr functionPointer = GetProcAddress(
      hModule, "CountClipboardFormats");

    // The returned function pointer returns an integer and
    // has no parameters, so invoke the function with our helper method
    int returnValue = (int)InvokeFunctionWithNoParams(
      functionPointer, typeof(int));

    // Print the result from calling CountClipboardFormats
    // via the function pointer
    Console.WriteLine("CountClipboardFormats returned: " + returnValue);
  }

  // Helper method that generates a dynamic method capable of invoking a
  // function pointer (via the calli instruction), then calls the dynamic
  // method and returns the result from the unmanaged function.
  public static object InvokeFunctionWithNoParams(IntPtr functionPointer,
    Type returnType)
  {
    // Create a dynamic assembly and a dynamic module
    AssemblyName asmName = new AssemblyName();
    asmName.Name = "tempAssembly";
    AssemblyBuilder dynamicAsm = 
      AppDomain.CurrentDomain.DefineDynamicAssembly(asmName, 
      AssemblyBuilderAccess.Run);
    ModuleBuilder dynamicMod = 
      dynamicAsm.DefineDynamicModule("tempModule");

    // Create a global method capable of invoking the function pointer
    MethodBuilder dynamicMethod = dynamicMod.DefineGlobalMethod(
      "DoTheDirtyWork", MethodAttributes.Public | 
      MethodAttributes.Static, returnType, null);

    // Generate the MSIL for this method
    ILGenerator generator = dynamicMethod.GetILGenerator();

    if (IntPtr.Size == 4)
      generator.Emit(OpCodes.Ldc_I4, functionPointer.ToInt32());
    else if (IntPtr.Size == 8)
      generator.Emit(OpCodes.Ldc_I8, functionPointer.ToInt64());
    else
      throw new PlatformNotSupportedException();

    // If the method had parameters, we'd push them on the stack first 
    generator.EmitCalli(OpCodes.Calli, CallingConvention.StdCall,
      returnType, new Type[]{});
    generator.Emit(OpCodes.Ret);
 
    // This global method is now complete
    dynamicMod.CreateGlobalFunctions();
 
    // Call the method we just created and return whatever it returns
    MethodInfo mi = dynamicMod.GetMethod("DoTheDirtyWork");
    return mi.Invoke(null, null);
  }
}