' To compile:
' vbc Listing19_3vb.vb

Imports System
Imports System.Reflection
Imports System.Reflection.Emit
Imports System.Runtime.InteropServices

Public Class InvokeFunctionPointer
  
  Declare Auto Function LoadLibrary Lib "kernel32.dll" (lpFileName As String) As IntPtr
  
  Declare Function GetProcAddress Lib "kernel32.dll" (hModule As IntPtr, lpProcName As String) As IntPtr
  
  Public Shared Sub Main()
    ' Call any unmanaged method that returns a function pointer.
    ' GetProcAddress is a canonical example.
    Dim hModule As IntPtr = LoadLibrary("user32.dll")
    Dim functionPointer As IntPtr = GetProcAddress(hModule, "CountClipboardFormats")
      
    ' The returned function pointer returns an integer and
    ' has no parameters, so invoke the function with our helper method
    Dim returnValue As Integer = CInt(InvokeFunctionWithNoParams(functionPointer, GetType(Integer)))
      
    ' Print the result from calling CountClipboardFormats
    ' via the function pointer
    Console.WriteLine("CountClipboardFormats returned: " & returnValue)
  End Sub
  
  ' Helper method that generates a dynamic method capable of invoking a
  ' function pointer (via the calli instruction), then calls the dynamic
  ' method and returns the result from the unmanaged function.
  Public Shared Function InvokeFunctionWithNoParams(functionPointer As IntPtr, returnType As Type) As Object
    ' Create a dynamic assembly and a dynamic module
    Dim asmName As New AssemblyName()
    asmName.Name = "tempAssembly"
    Dim dynamicAsm As AssemblyBuilder = AppDomain.CurrentDomain.DefineDynamicAssembly(asmName, AssemblyBuilderAccess.Run)
    Dim dynamicMod As ModuleBuilder = dynamicAsm.DefineDynamicModule("tempModule")
    
    ' Create a global method capable of invoking the function pointer
    Dim dynamicMethod As MethodBuilder = dynamicMod.DefineGlobalMethod("DoTheDirtyWork", MethodAttributes.Public Or MethodAttributes.Static, returnType, Nothing)
      
    ' Generate the MSIL for this method
    Dim generator As ILGenerator = dynamicMethod.GetILGenerator()
      
    If IntPtr.Size = 4 Then
      generator.Emit(OpCodes.Ldc_I4, functionPointer.ToInt32())
    ElseIf IntPtr.Size = 8 Then
      generator.Emit(OpCodes.Ldc_I8, functionPointer.ToInt64())
    Else
      Throw New PlatformNotSupportedException()
    End If 

    ' If the method had parameters, we'd push them on the stack first 
    generator.EmitCalli(OpCodes.Calli, CallingConvention.StdCall, returnType, New Type(){})
    generator.Emit(OpCodes.Ret)
    
    ' This global method is now complete
    dynamicMod.CreateGlobalFunctions()
    
    ' Call the method we just created and return whatever it returns
    Dim mi As MethodInfo = dynamicMod.GetMethod("DoTheDirtyWork")
    InvokeFunctionWithNoParams = mi.Invoke(Nothing, Nothing)
  End Function

End Class