' To compile:
' vbc /t:library Listing19_4vb.vb

Imports System
Imports System.Runtime.InteropServices

' Structs used by the PInvoke methods:
   
Structure CONSOLE_SCREEN_BUFFER_INFO 
  Public dwSize As COORD 
  Public dwCursorPosition As COORD 
  Public wAttributes As Short 
  Public srWindow As SMALL_RECT 
  Public dwMaximumWindowSize As COORD 
End Structure

Structure COORD 
  Public X As Short
  Public Y As Short
End Structure

Structure SMALL_RECT 
  Public Left As Short 
  Public Top As Short 
  Public Right As Short 
  Public Bottom As Short 
End Structure

' Constants used with PInvoke methods
Class Constants

  ' Control foreground color
  Public Const FOREGROUND_BLUE As Short      = &H01 
  Public Const FOREGROUND_GREEN As Short     = &H02
  Public Const FOREGROUND_RED As Short       = &H04
  Public Const FOREGROUND_INTENSITY As Short = &H08

  ' Control background color
  Public Const BACKGROUND_BLUE As Short      = &H10
  Public Const BACKGROUND_GREEN As Short     = &H20
  Public Const BACKGROUND_RED As Short       = &H40
  Public Const BACKGROUND_INTENSITY As Short = &H80

  ' Standard input, output, and error
  Public Const STD_INPUT_HANDLE As Integer  = -10
  Public Const STD_OUTPUT_HANDLE As Integer = -11
  Public Const STD_ERROR_HANDLE As Integer  = -12

  ' Returned by GetStdHandle when an error occurs
  Public Shared ReadOnly INVALID_HANDLE_VALUE As IntPtr = New IntPtr(-1)

End Class

' The public ColorfulConsole class
Public Class ColorfulConsole

  ' Three PInvoke signatures:

  Private Declare Function GetStdHandle Lib "kernel32.dll" (nStdHandle As Integer) As IntPtr

  Private Declare Function GetConsoleScreenBufferInfo Lib "kernel32.dll" _
    (hConsoleOutput As IntPtr, <Out> ByRef lpConsoleScreenBufferInfo As CONSOLE_SCREEN_BUFFER_INFO) As Boolean

  Private Declare Function SetConsoleTextAttribute Lib "kernel32.dll" _
    (hConsoleOutput As IntPtr, wAttributes As Short) As Boolean

  Private Shared csbi As CONSOLE_SCREEN_BUFFER_INFO = New CONSOLE_SCREEN_BUFFER_INFO()

  ' Two overloads of WriteLine:

  ' Print text with the given foreground color and background color
  Public Shared Sub WriteLine(text As String, foreColor As ForegroundColors, _
    backColor As BackgroundColors)

    Dim stdout As IntPtr = GetStdHandle(Constants.STD_OUTPUT_HANDLE)

    If (stdout.Equals(Constants.INVALID_HANDLE_VALUE)) Then
      Throw New Exception("Unable to get standard output handle")
    End If

    ' Save existing console settings
    If Not GetConsoleScreenBufferInfo(stdout, csbi) Then
      Throw New Exception("Unable to get existing console settings")
    End If

    ' Set the color
    If Not SetConsoleTextAttribute(stdout, foreColor Or backColor) Then
      Throw New Exception("Unable to set console colors")
    End If

    ' Write the text using System.Console
    Console.WriteLine(text)

    ' Restore the old console settings
    If Not SetConsoleTextAttribute(stdout, csbi.wAttributes) Then
      Throw New Exception("Unable to restore normal console colors")
    End If
  End Sub

  ' Print text with the given foreground color 
  ' and current console background color
  Public Shared Sub WriteLine(text As String, foreColor As ForegroundColors)

    Dim stdout As IntPtr = GetStdHandle(Constants.STD_OUTPUT_HANDLE)

    If (stdout.Equals(Constants.INVALID_HANDLE_VALUE)) Then
      Throw New Exception("Unable to get standard output handle")
    End If

    ' Retrieve the current background color
    If Not GetConsoleScreenBufferInfo(stdout, csbi) Then
      Throw New Exception("Unable to get existing console settings")
    End If

    Dim defaultBackground As BackgroundColors = csbi.wAttributes And &HF0

    WriteLine(text, foreColor, defaultBackground)
  End Sub

End Class

' Enumeration of foreground colors
Public Enum ForegroundColors As Short
  Black = 0
  Blue = Constants.FOREGROUND_BLUE
  Green = Constants.FOREGROUND_GREEN
  Cyan = Constants.FOREGROUND_BLUE Or Constants.FOREGROUND_GREEN
  Red = Constants.FOREGROUND_RED
  Magenta = Constants.FOREGROUND_RED Or Constants.FOREGROUND_BLUE
  Brown = Constants.FOREGROUND_RED Or Constants.FOREGROUND_GREEN
  Gray = Constants.FOREGROUND_RED Or Constants.FOREGROUND_BLUE Or _
    Constants.FOREGROUND_GREEN
  DarkGray = Constants.FOREGROUND_INTENSITY
  LightBlue = Constants.FOREGROUND_BLUE Or Constants.FOREGROUND_INTENSITY
  LightGreen = Constants.FOREGROUND_GREEN Or _
    Constants.FOREGROUND_INTENSITY
  LightCyan = Constants.FOREGROUND_BLUE Or Constants.FOREGROUND_GREEN Or _
    Constants.FOREGROUND_INTENSITY
  LightRed = Constants.FOREGROUND_RED Or Constants.FOREGROUND_INTENSITY
  LightMagenta = Constants.FOREGROUND_RED Or Constants.FOREGROUND_BLUE Or _
    Constants.FOREGROUND_INTENSITY
  Yellow = Constants.FOREGROUND_RED Or Constants.FOREGROUND_GREEN Or _
    Constants.FOREGROUND_INTENSITY
  White = Constants.FOREGROUND_RED Or Constants.FOREGROUND_BLUE Or _
    Constants.FOREGROUND_GREEN Or Constants.FOREGROUND_INTENSITY
End Enum

' Enumeration of background colors
Public Enum BackgroundColors As Short
  Black = 0
  Blue = Constants.BACKGROUND_BLUE
  Green = Constants.BACKGROUND_GREEN
  Cyan = Constants.BACKGROUND_BLUE Or Constants.BACKGROUND_GREEN
  Red = Constants.BACKGROUND_RED
  Magenta = Constants.BACKGROUND_RED Or Constants.BACKGROUND_BLUE
  Brown = Constants.BACKGROUND_RED Or Constants.BACKGROUND_GREEN
  Gray = Constants.BACKGROUND_RED Or Constants.BACKGROUND_BLUE Or _
    Constants.BACKGROUND_GREEN
  DarkGray = Constants.BACKGROUND_INTENSITY
  LightBlue = Constants.BACKGROUND_BLUE Or Constants.BACKGROUND_INTENSITY
  LightGreen = Constants.BACKGROUND_GREEN Or _
    Constants.BACKGROUND_INTENSITY
  LightCyan = Constants.BACKGROUND_BLUE Or Constants.BACKGROUND_GREEN Or _
    Constants.BACKGROUND_INTENSITY
  LightRed = Constants.BACKGROUND_RED Or Constants.BACKGROUND_INTENSITY
  LightMagenta = Constants.BACKGROUND_RED Or Constants.BACKGROUND_BLUE Or _
    Constants.BACKGROUND_INTENSITY
  Yellow = Constants.BACKGROUND_RED Or Constants.BACKGROUND_GREEN Or _
    Constants.BACKGROUND_INTENSITY
  White = Constants.BACKGROUND_RED Or Constants.BACKGROUND_BLUE Or _
    Constants.BACKGROUND_GREEN Or Constants.BACKGROUND_INTENSITY
End Enum