// To compile:
// cl /CLR DisposableMarshaler.cpp

#using <mscorlib.dll> // Contains managed definition of UCOMIPersistFile
#include <objidl.h>   // Contains unmanaged definition of IPersistFile
#include <vcclr.h>    // Contains the gcroot template

using namespace System;
using namespace System::Runtime::InteropServices;

// CustomCCW - A Custom COM-Callable Wrapper
class CustomCCW : public IPersistFile
{
public:

  // Constructor
  CustomCCW (UCOMIPersistFile *obj)
  {
    originalObject = obj;
    referenceCount = 0;
  };

  // IUnknown.QueryInterface
  HRESULT STDMETHODCALLTYPE QueryInterface(const IID& iid, void** ppv)
  {
    if (iid == IID_IUnknown)
    {
      *ppv = static_cast<IPersistFile*>(this);
    }
    else if (iid == IID_IPersistFile)
    {
      *ppv = static_cast<IPersistFile*>(this);
    }
    else
    {
      *ppv = NULL;
      return E_NOINTERFACE;
    }
    static_cast<IUnknown*>(*ppv)->AddRef();
    return S_OK;
  };

  // IUnknown.AddRef
  ULONG STDMETHODCALLTYPE AddRef()
  {
    InterlockedIncrement(&referenceCount);
    return referenceCount;
  };
  
  // IUnknown.Release
  ULONG STDMETHODCALLTYPE Release()
  {
    InterlockedDecrement(&referenceCount);
    if (referenceCount == 0)
    {
      // Dispose the .NET object when the reference count reaches zero
      try
      {
        IDisposable *d = __try_cast<IDisposable*>
          ((UCOMIPersistFile*)originalObject);
        d->Dispose();
      }
      catch (InvalidCastException*) {}

      delete this;
      return 0;
    }
    return referenceCount;
  };

  // Implementation of the unmanaged IPersistFile methods, which do the
  // appropriate marshaling then call the corresponding managed methods

  HRESULT STDMETHODCALLTYPE GetClassID(CLSID *pClassID) 
  { 
    try
    {
      Guid* g = __nogc new Guid(pClassID->Data1, pClassID->Data2, 
        pClassID->Data3, pClassID->Data4[0], pClassID->Data4[1], 
        pClassID->Data4[2], pClassID->Data4[3], pClassID->Data4[4],
        pClassID->Data4[5], pClassID->Data4[6], pClassID->Data4[7]);

      originalObject->GetClassID(g); 
    }
    catch (Exception* ex)
    {
      return Marshal::GetHRForException(ex);
    }
  };

  HRESULT STDMETHODCALLTYPE IsDirty() 
  { 
    return originalObject->IsDirty(); 
  };

  HRESULT STDMETHODCALLTYPE Load(LPCOLESTR pszFileName, DWORD dwMode) 
  { 
    try
    {
      String* s = Marshal::PtrToStringUni((IntPtr)(int)pszFileName);
      originalObject->Load(s, dwMode); 
    }
    catch (Exception* ex)
    {
      return Marshal::GetHRForException(ex);
    }
  };

  HRESULT STDMETHODCALLTYPE Save(LPCOLESTR pszFileName, BOOL fRemember)
  { 
    try
    {
      String* s = Marshal::PtrToStringUni(
        __nogc new IntPtr((void*)pszFileName));
      originalObject->Save(s, fRemember);
    }
    catch (Exception* ex)
    {
      return Marshal::GetHRForException(ex);
    }
  };

  HRESULT STDMETHODCALLTYPE SaveCompleted(LPCOLESTR pszFileName)
  { 
    try
    {
      String* s = Marshal::PtrToStringUni(
        __nogc new IntPtr((void*)pszFileName));
      originalObject->SaveCompleted(pszFileName); 
    }
    catch (Exception* ex)
    {
      return Marshal::GetHRForException(ex);
    }
  };

  HRESULT STDMETHODCALLTYPE GetCurFile(LPOLESTR *ppszFileName)
  { 
    try
    {
      String* s = Marshal::PtrToStringUni(
        __nogc new IntPtr(*ppszFileName));
      originalObject->GetCurFile(&s);
    }
    catch (Exception* ex)
    {
      return Marshal::GetHRForException(ex);
    }
  };

private:
  gcroot<UCOMIPersistFile*> originalObject;
  LONG referenceCount;
};

public __gc class DisposableMarshaler : public ICustomMarshaler
{
private:
  static DisposableMarshaler *marshaler = NULL;

public:
  Object* MarshalNativeToManaged(IntPtr pNativeData)
  {
    if (pNativeData == IntPtr::Zero) return NULL;

    // Just use the standard RCW
    return Marshal::GetObjectForIUnknown(pNativeData);
  };

  IntPtr MarshalManagedToNative(Object* ManagedObj)
  {
    if (ManagedObj == NULL) return IntPtr::Zero;
    CustomCCW *ccw = new CustomCCW(static_cast<UCOMIPersistFile*>(ManagedObj));
    IPersistFile* intf = NULL;
    ccw->QueryInterface(IID_IPersistFile, (void**)&intf);
    return (IntPtr)intf;
  };
 
  void CleanUpNativeData(IntPtr pNativeData)
  {
    Marshal::Release(pNativeData);
  };

  void CleanUpManagedData(Object* ManagedObj)
  {
    // Nothing to do
  };

  int GetNativeDataSize()
  {
    return -1;
  };

  static ICustomMarshaler* GetInstance(String* cookie)
  {
    // Always return the same instance
    if (marshaler != NULL) marshaler = new DisposableMarshaler();
    return marshaler;
  };
};