' To compile, first compile IFont.cs.  Then run:
' vbc /t:library /out:ManagedForm.dll ManagedForm.vb FontMarshaler.vb /r:System.Windows.Forms.dll /r:System.dll /r:System.Drawing.dll /r:IFont.dll

Imports System
Imports System.Drawing
Imports System.Runtime.InteropServices

' The Font Custom Marshaler
Public Class FontMarshaler
  Implements ICustomMarshaler

  Private Shared marshaler As ICustomMarshaler 

  Public Function MarshalNativeToManaged(pNativeData As IntPtr) _
    As Object Implements ICustomMarshaler.MarshalNativeToManaged
    If (pNativeData.Equals(IntPtr.Zero)) Then
      MarshalNativeToManaged = Nothing
    End If

    ' Extract the IFont interface
    Dim unmanagedFont As IFont = _
      CType(Marshal.GetObjectForIUnknown(pNativeData), IFont)

    ' Create a .NET Font from IFont's hFont property.  This throws
    ' an ArgumentException if the unmanaged font isn't TrueType
    MarshalNativeToManaged = Font.FromHFont(unmanagedFont.hFont)
  End Function

  Public Function MarshalManagedToNative(ManagedObj As Object) _
    As IntPtr Implements ICustomMarshaler.MarshalManagedToNative
    If (ManagedObj Is Nothing) Then
      MarshalManagedToNative = IntPtr.Zero
    End If

    Dim managedFont As Font = ManagedObj
    Dim log As New LOGFONT
    managedFont.ToLogFont(log)

    ' Extract information about the .NET font 
    Dim desc As FONTDESC
    desc.cbSizeofstruct = Marshal.SizeOf(desc)
    desc.lpstrName = managedFont.Name
    desc.cySize = managedFont.SizeInPoints * 10000
    desc.sWeight = log.lfWeight
    desc.sCharset = log.lfCharSet
    desc.fItalic = managedFont.Italic
    desc.fUnderline = managedFont.Underline
    desc.fStrikethrough = managedFont.Strikeout

    ' Create a StdFont object using the FONTDESC
    Dim unmanagedFont As IFont
    Dim hresult As Integer = _
      OleCreateFontIndirect(desc, GetType(IFont).GUID, unmanagedFont)
    If (hresult < 0) Then Marshal.ThrowExceptionForHR(hresult)

    MarshalManagedToNative = _ 
      Marshal.GetComInterfaceForObject(unmanagedFont, GetType(IFont))
  End Function

  Public Sub CleanUpNativeData(pNativeData As IntPtr) _
    Implements ICustomMarshaler.CleanUpNativeData
    ' Call release twice due to two AddRefs in MarshalManagedToNative
    Marshal.Release(pNativeData)
    Marshal.Release(pNativeData)
  End Sub

  Public Sub CleanUpManagedData(ManagedObj As Object) _
    Implements ICustomMarshaler.CleanUpManagedData
    ' Font implements IDisposable
    CType(ManagedObj, IDisposable).Dispose()
  End Sub

  Public Function GetNativeDataSize() As Integer _
    Implements ICustomMarshaler.GetNativeDataSize
    GetNativeDataSize = -1
  End Function

  Public Shared Function GetInstance(cookie As String) As ICustomMarshaler
    If (marshaler Is Nothing) Then 
      marshaler = New FontMarshaler()
    End If
    GetInstance = marshaler
  End Function

  ' Used to create a StdFont object that implements IFont
  Declare Function OleCreateFontIndirect Lib "oleaut32.dll" _
    (<[In]> ByRef fontdesc As FONTDESC, <[In]> ByRef refiid As Guid, _
    <Out, MarshalAs(UnmanagedType.Interface)> ByRef ppvObj As Object) _
    As Integer  
End Class

Public Structure FONTDESC
  Public cbSizeofstruct As Integer
  <MarshalAs(UnmanagedType.LPWStr)> Public lpstrName As String
  Public cySize As Long
  Public sWeight As Short
  Public sCharset As Short
  Public fItalic As Boolean
  Public fUnderline As Boolean
  Public fStrikethrough As Boolean
End Structure

<StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
Public Class LOGFONT
  Public lfHeight As Integer
  Public lfWidth As Integer
  Public lfEscapement As Integer
  Public lfOrientation As Integer
  Public lfWeight As Integer
  Public lfItalic As Byte
  Public lfUnderline As Byte
  Public lfStrikeOut As Byte
  Public lfCharSet As Byte
  Public lfOutPrecision As Byte
  Public lfClipPrecision As Byte
  Public lfQuality As Byte
  Public lfPitchAndFamily As Byte
  <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=32)> _
  Public lfFaceName As String
End Class