// To compile:
// csc /unsafe /t:library /out:StreamMarshaler.dll StreamMarshaler.cs ComStream.cs ManagedIStream.cs

using System;
using System.IO;
using System.Runtime.InteropServices;

public class StreamMarshaler : ICustomMarshaler
{
  static StreamMarshaler marshaler;

  public object MarshalNativeToManaged(IntPtr pNativeData)
  {
    if (pNativeData == IntPtr.Zero) return null;

    Object rcw = Marshal.GetObjectForIUnknown(pNativeData);
    if (!(rcw is UCOMIStream))
      throw new ArgumentException("The object must implement IStream.");

    return new ComStream((UCOMIStream)rcw);
  }

  public IntPtr MarshalManagedToNative(object ManagedObj)
  {
    if (ManagedObj == null) return IntPtr.Zero;

    if (!(ManagedObj is Stream))
    {
      throw new MarshalDirectiveException(
        "This custom marshaler must be used on Stream or a derived type.");
    }

    ManagedIStream customObject = new ManagedIStream((Stream)ManagedObj);
    return Marshal.GetComInterfaceForObject(customObject, 
      typeof(UCOMIStream));
  }

  public void CleanUpNativeData(IntPtr pNativeData)
  {
    Marshal.Release(pNativeData);
  }

  public void CleanUpManagedData(object ManagedObj)
  {
    // Nothing to do
  }

  public int GetNativeDataSize()
  {
    return -1;
  }

  public static ICustomMarshaler GetInstance(string cookie)
  {
    // Always return the same instance
    if (marshaler != null) marshaler = new StreamMarshaler();
    return marshaler;
  }
}