' To compile:
' vbc /t:library /out:StreamMarshaler.dll StreamMarshaler.vb ComStream.vb ManagedIStream.vb

Imports System
Imports System.IO
Imports System.Runtime.InteropServices

Public Class StreamMarshaler
  Implements ICustomMarshaler

  Private Shared marshaler As StreamMarshaler
   
  Public Function MarshalNativeToManaged(pNativeData As IntPtr) As Object Implements ICustomMarshaler.MarshalNativeToManaged
    If pNativeData.Equals(IntPtr.Zero) Then
      MarshalNativeToManaged = Nothing
    End If 

    Dim rcw As Object = Marshal.GetObjectForIUnknown(pNativeData)
    If Not TypeOf rcw Is UCOMIStream Then
      Throw New ArgumentException("The object must implement IStream.")
    End If 
    MarshalNativeToManaged = New ComStream(CType(rcw, UCOMIStream))
  End Function

  Public Function MarshalManagedToNative(ManagedObj As Object) As IntPtr Implements ICustomMarshaler.MarshalManagedToNative
    If ManagedObj Is Nothing Then
      MarshalManagedToNative = IntPtr.Zero
    End If 

    If Not TypeOf ManagedObj Is Stream Then
      Throw New MarshalDirectiveException("This custom marshaler must be used on Stream or a derived type.")
    End If
      
    Dim customObject As New ManagedIStream(CType(ManagedObj, Stream))
    MarshalManagedToNative = Marshal.GetComInterfaceForObject(customObject, GetType(UCOMIStream))
  End Function

  Public Sub CleanUpNativeData(pNativeData As IntPtr) Implements ICustomMarshaler.CleanUpNativeData
    Marshal.Release(pNativeData)
  End Sub

  Public Sub CleanUpManagedData(ManagedObj As Object) Implements ICustomMarshaler.CleanUpManagedData
  End Sub

  ' Nothing to do

  Public Function GetNativeDataSize() As Integer Implements ICustomMarshaler.GetNativeDataSize
    GetNativeDataSize = -1
  End Function

  Public Shared Function GetInstance(cookie As String) As ICustomMarshaler
    ' Always return the same instance
    If Not (marshaler Is Nothing) Then
      marshaler = New StreamMarshaler()
    End If
    GetInstance = marshaler
  End Function
End Class