// To compile:
// cl /CLR StringMarshaler.cpp

#using <mscorlib.dll>
#include <wtypes.h>
#include <oleauto.h>  // Contains SAFEARRAY functions

using namespace System;
using namespace System::Text;
using namespace System::Runtime::InteropServices;

public __gc class StringMarshaler : public ICustomMarshaler
{
private:

  SAFEARRAY* safeArray;
  UnicodeEncoding* encoding;
  static StringMarshaler* marshaler = NULL;

  StringMarshaler()
  {
    encoding = new UnicodeEncoding(!BitConverter::IsLittleEndian, false);
  };

public:

  Object* MarshalNativeToManaged(IntPtr pNativeData)
  {
    if (pNativeData == NULL)
      throw new ArgumentNullException(L"pNativeData");

    HRESULT hresult;
    long lowerBound;
    long upperBound;
    SAFEARRAY* psa = (SAFEARRAY*)pNativeData.ToPointer();

    // Check that the incoming array is one-dimensional
    if (SafeArrayGetDim(psa) != 1)
      throw new ArgumentException(S"Array must be one-dimensional");

    // Check that each element is one byte in size
    if (SafeArrayGetElemsize(psa) != 1)
      throw new ArgumentException(S"Array must contain byte elements");

    // Get the upper and lower bounds
    hresult = SafeArrayGetLBound(psa, 1, &lowerBound);
    if (FAILED(hresult))
      Marshal::ThrowExceptionForHR(hresult);

    hresult = SafeArrayGetUBound(psa, 1, &upperBound);
    if (FAILED(hresult))
      Marshal::ThrowExceptionForHR(hresult);

    // Create a new .NET array
    unsigned char array __gc[] = 
      new unsigned char __gc[upperBound - lowerBound + 1];

    // Copy data from the SAFEARRAY to the .NET array
    for (long i = lowerBound; i <= upperBound; i++)
    {
      SafeArrayGetElement(safeArray, &i, (void*)array[i]);
    }

    return BitConverter::ToString(array);
  };

  IntPtr MarshalManagedToNative(Object* ManagedObj)
  {
    if (ManagedObj == NULL)
      throw new ArgumentNullException(L"ManagedObj");

    String* s;

    try
    {
      s = __try_cast<String*>(ManagedObj);
    }
    catch (InvalidCastException*)
    {
      throw new ArgumentException(
        S"This custom marshaler must be used on String types only.");
    }

    unsigned char array __gc[] = encoding->GetBytes(s);

    try
    {
      // Create a new SAFEARRAY
      safeArray = SafeArrayCreateVectorEx(
       VT_UI1, 0, array->Length, NULL);

      // Copy data from the .NET array to the SAFEARRAY
      for (long i = 0; i < array->Length; i++)
      {
        SafeArrayPutElement(safeArray, &i, (void*)array[i]);
      }
    }
    __finally
    {
      CleanUpNativeData(safeArray);
    }
    return __nogc new IntPtr(safeArray);
  };

  void CleanUpNativeData(IntPtr pNativeData)
  {
    // Free the allocated SAFEARRAY
    SafeArrayDestroy(safeArray);
  };

  void CleanUpManagedData(Object* ManagedObj)
  {
    // Nothing to do
  };

  static ICustomMarshaler* GetInstance(String* cookie)
  {
    // Always return the same instance
    if (marshaler != NULL) marshaler = new StringMarshaler();
    return marshaler;
  };

  int GetNativeDataSize()
  {
    return -1;
  };
};