// To compile:
// csc /t:library Listing21_18cs.cs

using System;
using System.Runtime.InteropServices;

[
  ComImport,
  Guid("D3984C13-C3CB-48E2-8BE5-5168340B4F35"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
interface IWMPEffects
{
  void Render(ref TimedLevel levels, IntPtr hdc, ref RECT r);
  void MediaInfo(int channelCount, int sampleRate, string title);
  VisualizationCapabilities Capabilities { get; }
  string Title { get; }
  string GetPresetTitle(int preset);
  int PresetCount { get; }
  int CurrentPreset { set; get; }
  void DisplayPropertyPage(IntPtr hwndOwner);
  void GoFullScreen(
    [MarshalAs(UnmanagedType.Bool)] bool startFullScreen);
  void RenderFullScreen(ref TimedLevel levels);
}

// A visualization's capabilities
[ComVisible(false), Flags]
public enum VisualizationCapabilities : uint
{
  None = 0,
  CanGoFullScreen = 1,
  HasPropertyPage = 2,
  VariableFrequencyStep = 4,
}

// The structure holding the frequency, waveform and 
// state data crucial for rendering
[ComVisible(false), StructLayout(LayoutKind.Sequential, Pack=4)]
public struct TimedLevel
{
  public const int MaxSamples = 1024;  

  [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2*MaxSamples)]
  public byte [] frequency;      

  [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2*MaxSamples)]
  public byte [] waveform;       

  public AudioState State;
  public long TimeStamp;
}

// A standard Windows rectangle
[ComVisible(false), StructLayout(LayoutKind.Sequential, Pack=4)]
public struct RECT 
{
  public int Left;
  public int Top;
  public int Right;
  public int Bottom;
}

// The state of the audio
[ComVisible(false)]
public enum AudioState       
{
  Stopped = 0,
  Paused  = 1,
  Playing = 2
}