// To compile:
// csc /t:library Listing21_21.cs SimpleClasses.cs

using System;
using System.Collections;
using System.Runtime.InteropServices;

// Coclass Interface
[
  ComImport,
  Guid("5572984E-7A76-11D2-9334-0000F875AE17"),
  CoClass(typeof(NetMeetingClass))
]
public interface NetMeeting : INetMeeting, _INetMeetingEvents_Event {}

// Raw Source Interface
[
  ComImport,
  Guid("3E9BAF2C-7A79-11D2-9334-0000F875AE17"),
  InterfaceType(ComInterfaceType.InterfaceIsIDispatch)
]
public interface _INetMeetingEvents 
{
  [DispId(202)]
  void ConferenceStarted();
  [DispId(203)]
  void ConferenceEnded();
}

// Default Interface
[
  ComImport,
  Guid("5572984E-7A76-11D2-9334-0000F875AE17"),
  TypeLibType(TypeLibTypeFlags.FDual | TypeLibTypeFlags.FHidden | 
    TypeLibTypeFlags.FNonExtensible)
]
public interface INetMeeting 
{
  [DispId(100)]
  int Version();
  [DispId(101)]
  void UnDock();
  [DispId(102)]
  [return: MarshalAs(UnmanagedType.Bool)] bool IsInConference();
  [DispId(103)]
  void CallTo(string strCallToString);
  [DispId(104)]
  void LeaveConference();
}

// Event Interface
[
  ComVisible(false),
  ComEventInterfaceAttribute(typeof(_INetMeetingEvents), 
    typeof(_INetMeetingEvents_EventProvider))
]
public interface _INetMeetingEvents_Event 
{
  event _INetMeetingEvents_ConferenceEndedEventHandler ConferenceEnded;
  event _INetMeetingEvents_ConferenceStartedEventHandler 
    ConferenceStarted;
}

// Delegates, one for each event
[ComVisible(false)]
public delegate void _INetMeetingEvents_ConferenceEndedEventHandler();

[ComVisible(false)]
public delegate void _INetMeetingEvents_ConferenceStartedEventHandler();

// Sink Helper Class
[ClassInterface(ClassInterfaceType.None)]
sealed class _INetMeetingEvents_SinkHelper : _INetMeetingEvents
{
  // Delegates, one per method
  public _INetMeetingEvents_ConferenceEndedEventHandler 
    m_ConferenceEndedDelegate;
  public _INetMeetingEvents_ConferenceStartedEventHandler 
    m_ConferenceStartedDelegate;

  // The connection point cookie from Advise, saved for Unadvise
  public int m_dwCookie;

  // Constructor
  internal _INetMeetingEvents_SinkHelper()
  {
    m_dwCookie = 0;
    m_ConferenceEndedDelegate = null;
    m_ConferenceStartedDelegate = null;
  }

  public void ConferenceEnded()
  {
    // Invoke the corresponding delegate
    if (m_ConferenceEndedDelegate != null)
      m_ConferenceEndedDelegate();
  }

  public void ConferenceStarted()
  {
    // Invoke the corresponding delegate
    if (m_ConferenceStartedDelegate != null)
      m_ConferenceStartedDelegate();
  }
}

// Event Provider Class
sealed class _INetMeetingEvents_EventProvider : _INetMeetingEvents_Event,
  IDisposable
{
  private UCOMIConnectionPointContainer m_ConnectionPointContainer;
  private UCOMIConnectionPoint m_ConnectionPoint;
  private ArrayList m_aEventSinkHelpers;

  // Constructor
  public _INetMeetingEvents_EventProvider(object container)
  {
    m_ConnectionPointContainer = (UCOMIConnectionPointContainer)container;
  }

  // Enables lazy initialization of connection point.
  // Called the first time someone hooks up to an event.
  private void Init()
  {
    Guid iid = typeof(_INetMeetingEvents).GUID;
    m_ConnectionPointContainer.FindConnectionPoint(ref iid, 
      out m_ConnectionPoint);
    m_aEventSinkHelpers = new ArrayList();
  }

  // Implementation of IDisposable.Dispose
  public void Dispose()
  {
    Cleanup();
    // It's no longer necessary for the garbage collector 
    // to call the finalizer
    GC.SuppressFinalize(this);
  }

  // Finalizer
  ~_INetMeetingEvents_EventProvider()
  {
    Cleanup();
  }

  // Called by Dispose and Finalizer to unhook all events
  private void Cleanup()
  {
   _INetMeetingEvents_SinkHelper sinkHelper;

    if (m_ConnectionPoint != null)
    {
      lock (this)
      {
        try
        {
          // Go through all sink helpers and "unhook" them
          for (int i = 0; i < m_aEventSinkHelpers.Count; i++)
          {
            // Remove the sink helper from the list and call Unadvise
            sinkHelper = (_INetMeetingEvents_SinkHelper)
              m_aEventSinkHelpers[i];
            m_ConnectionPoint.Unadvise(sinkHelper.m_dwCookie);
          } 
        } catch (Exception) 
        {
          // Swallow exceptions
        }
      }
    }
  }

  // Event #1: ConferenceEnded
  public event _INetMeetingEvents_ConferenceEndedEventHandler 
  ConferenceEnded
  {
    add 
    {
      _INetMeetingEvents_SinkHelper sinkHelper;

      lock (this)
      {
        if (m_ConnectionPoint == null)
          Init();

        // Call Advise on a new sink helper, initializing its cookie
        sinkHelper = new _INetMeetingEvents_SinkHelper();
        m_ConnectionPoint.Advise(sinkHelper, out sinkHelper.m_dwCookie);

        // Initialize the sink helper's delegate to the passed-in value
        sinkHelper.m_ConferenceEndedDelegate = value;

        // Add the sink helper to the list
        m_aEventSinkHelpers.Add(sinkHelper);
      }
    }
    remove
    {
      _INetMeetingEvents_SinkHelper sinkHelper;

      lock (this)
      {
        // Go through all sink helpers until we find the right one
        for (int i = 0; i < m_aEventSinkHelpers.Count; i++)
        {
          sinkHelper = (_INetMeetingEvents_SinkHelper)
            m_aEventSinkHelpers[i];

          // Check if this is the correct sink helper
          if (sinkHelper.m_ConferenceEndedDelegate != null &&
              sinkHelper.m_ConferenceEndedDelegate.Equals(value))
          {
            // Remove the sink helper from the list
            m_aEventSinkHelpers.RemoveAt(i);

            // Call Unadvise using the cookie saved from Advise
            m_ConnectionPoint.Unadvise(sinkHelper.m_dwCookie);

            // If this is the last sink helper in the list, set the
            // connection point and array list to null
            if (m_aEventSinkHelpers.Count == 0)
            {
              m_ConnectionPoint = null;
              m_aEventSinkHelpers = null;
            }
            break;
          }
        } // end for
      } // end lock
    } // end remove
  }

  // Event #2: ConferenceStarted
  public event _INetMeetingEvents_ConferenceStartedEventHandler 
  ConferenceStarted
  {
    add 
    {
      _INetMeetingEvents_SinkHelper sinkHelper;

      lock (this)
      {
        if (m_ConnectionPoint == null)
          Init();

        // Call Advise on a new sink helper, initializing its cookie
        sinkHelper = new _INetMeetingEvents_SinkHelper();
        m_ConnectionPoint.Advise(sinkHelper, out sinkHelper.m_dwCookie);

        // Initialize the sink helper's delegate to the passed-in value
        sinkHelper.m_ConferenceStartedDelegate = value;

        // Add the sink helper to the list
        m_aEventSinkHelpers.Add(sinkHelper);
      }
    }
    remove
    {
      _INetMeetingEvents_SinkHelper sinkHelper;

      lock (this)
      {
        // Go through all sink helpers until we find the right one
        for (int i = 0; i < m_aEventSinkHelpers.Count; i++)
        {
          sinkHelper = (_INetMeetingEvents_SinkHelper)
            m_aEventSinkHelpers[i];

          // Check if this is the correct sink helper
          if (sinkHelper.m_ConferenceStartedDelegate != null &&
              sinkHelper.m_ConferenceStartedDelegate.Equals(value))
          {
            // Remove the sink helper from the list
            m_aEventSinkHelpers.RemoveAt(i);

            // Call Unadvise using the cookie saved from Advise
            m_ConnectionPoint.Unadvise(sinkHelper.m_dwCookie);

            // If this is the last sink helper in the list, set the
            // connection point and array list to null
            if (m_aEventSinkHelpers.Count == 0)
            {
              m_ConnectionPoint = null;
              m_aEventSinkHelpers = null;
            }
            break;
          }
        } // end for
      } // end lock
    } // end remove
  }
}