// To compile:
// csc /t:library Listing21_4.cs

using System;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

[
  ComImport,
  Guid("D3984C13-C3CB-48E2-8BE5-5168340B4F35"),
  InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
]
interface IWMPEffects
{
  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void Render([In] ref TimedLevel pLevels, [In] IntPtr hdc,
    [In] ref RECT prc);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void MediaInfo([In] int lChannelCount, [In] int lSampleRate,
    [In, MarshalAs(UnmanagedType.BStr)] string bstrTitle);
 
  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void GetCapabilities(out VisualizationCapabilities pdwCapabilities);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void GetTitle([MarshalAs(UnmanagedType.BStr)] out string bstrTitle);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void GetPresetTitle([In] int nPreset, [MarshalAs(UnmanagedType.BStr)]
    out string bstrPresetTitle);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void GetPresetCount(out int pnPresetCount);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void SetCurrentPreset([In] int nPreset);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void GetCurrentPreset(out int pnPreset);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void DisplayPropertyPage([In] IntPtr hwndOwner);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void GoFullScreen([In, MarshalAs(UnmanagedType.Bool)] bool fFullScreen);

  [MethodImpl(MethodImplOptions.InternalCall, 
   MethodCodeType=MethodCodeType.Runtime)]
  void RenderFullScreen([In] ref TimedLevel pLevels);
}

// A visualization's capabilities
[ComVisible(false), Flags]
public enum VisualizationCapabilities : uint
{
  None = 0,
  CanGoFullScreen = 1,
  HasPropertyPage = 2,
  VariableFrequencyStep = 4,
}

// The structure holding the frequency, waveform and 
// state data crucial for rendering
[ComVisible(false), StructLayout(LayoutKind.Sequential, Pack=4)]
public struct TimedLevel
{
  public const int MaxSamples = 1024;  

  [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2*MaxSamples)]
  public byte [] frequency;      

  [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2*MaxSamples)]
  public byte [] waveform;       

  public AudioState State;
  public long TimeStamp;
}

// A standard Windows rectangle
[ComVisible(false), StructLayout(LayoutKind.Sequential, Pack=4)]
public struct RECT 
{
  public int Left;
  public int Top;
  public int Right;
  public int Bottom;
}

// The state of the audio
[ComVisible(false)]
public enum AudioState       
{
  Stopped = 0,
  Paused  = 1,
  Playing = 2
}