// To compile:
// csc /out:Interop.exe Importer.cs Exporter.cs Registrar.cs ComPlusInstaller.cs MainForm.cs MiscTypes.cs /r:System.Windows.Forms.dll /r:System.dll /r:System.Drawing.dll /r:System.EnterpriseServices.dll

using System.EnterpriseServices;

// COM+ Installation Options
public struct ComPlusInstallerOptions
{
  public string ApplicationName;
  public string PartitionName;
  public string TypeLibName;
  public InstallationFlags Flags;
}

// Installs and uninstalls assemblies for COM+
public class ComPlusInstaller
{
  private ComPlusInstallerOptions options;
  private RegistrationHelper registration;
  private IToolClient client;
  
  // Constructor
  public ComPlusInstaller(IToolClient client)
  {
    this.client = client;
    this.registration = new RegistrationHelper();
  }

  // Property to get/set options
  public ComPlusInstallerOptions Options
  {
    get { return options; }
    set { options = value; }
  }

  // Installs the assembly
  public string Install(string assemblyName)
  {
    registration.InstallAssembly(assemblyName, 
      ref options.ApplicationName, options.PartitionName, 
      ref options.TypeLibName, options.Flags);

    if (options.ApplicationName == null)
    {
      return "The assembly contains no ServicedComponent " +
        "types to register.";
    }

    if (options.PartitionName == null)
    {
      return "Successfully registered '" + assemblyName + 
        "' as the application '" + options.ApplicationName + 
        "' with type library '" + options.TypeLibName + "'.";
    }
    else
    {
      return "Successfully registered '" + assemblyName + 
        "' as the application '" + options.ApplicationName +
        "' with partition '" + options.PartitionName + 
        "' and type library '" + options.TypeLibName + "'.";
    }
  }

  // Uninstalls the assembly
  public string Uninstall(string assemblyName)
  {
    registration.UninstallAssembly(assemblyName, 
      options.ApplicationName, options.PartitionName);
    return "Successfully unregistered '" + assemblyName + "'.";
  }
}