// To compile:
// csc /out:Interop.exe Importer.cs Exporter.cs Registrar.cs ComPlusInstaller.cs MainForm.cs MiscTypes.cs /r:System.Windows.Forms.dll /r:System.dll /r:System.Drawing.dll /r:System.EnterpriseServices.dll

using System;
using System.Reflection;
using System.Collections;
using System.Runtime.InteropServices;

// Registration Options
public struct RegisterOptions
{
  public AssemblyRegistrationFlags Flags;
}

// Registers and unregisters assemblies for COM
public class Registrar
{
  private RegisterOptions options;
  private RegistrationServices registration;
  private IToolClient client;
  private Hashtable loadedAssemblies;
  
  // Constructor
  public Registrar(IToolClient client, Hashtable loadedAssemblies)
  {
    this.client = client;
    this.loadedAssemblies = loadedAssemblies;
    this.registration = new RegistrationServices();
  }

  // Property to get/set options
  public RegisterOptions Options
  {
    get { return options; }
    set { options = value; }
  }

  // Registers the assembly given by the filename
  public string Register(string assemblyName)
  {
    return Register(LoadAndDisplayAssembly(assemblyName));
  }

  // Unregisters the assembly given by the filename
  public string Unregister(string assemblyName)
  {
    return Unregister(LoadAndDisplayAssembly(assemblyName));
  }

  // Registers the loaded assembly
  public string Register(Assembly assembly)
  {
    try
    {
      // Register the assembly
      if (registration.RegisterAssembly(assembly, options.Flags))
        return "Types registered successfully.";
      else
        return "There were no types to register.";
    }
    catch (TargetInvocationException ex)
    {
      // Construct a better message for registration function error
      throw new Exception("Error from custom registration function: " +
        ex.InnerException.Message, ex.InnerException);
    }
  }

  // Unregisters the loaded assembly
  public string Unregister(Assembly assembly)
  {
    try
    {
      // Unregister the assembly
      if (registration.UnregisterAssembly(assembly))
        return "Types unregistered successfully.";
      else
        return "There were no types to unregister.";
    }
    catch (TargetInvocationException ex)
    {
      // Construct a better message for unregistration function error
      throw new Exception("Error from custom unregistration function: " +
        ex.InnerException.Message, ex.InnerException);
    }
  }

  // Loads an assembly from the input filename and notifies
  // the client so it can display it.
  private Assembly LoadAndDisplayAssembly(string assemblyName)
  {
    Assembly returnAssembly;

    try
    {
      // Load the assembly
      returnAssembly = Assembly.LoadFrom(assemblyName);
    }
    catch
    {
      throw new ApplicationException("Unable to load the assembly '" +
        assemblyName + "'.");
    }

    // Construct a nice "display name" for the assembly
    string assemblyDisplayName = "Assembly: " + 
      Util.GetDisplayNameForAssembly(returnAssembly);

    // Notify the client that the assembly has been loaded.
    // Give both beginning and ending notifications since there's no
    // progress to report.
    client.SendMessage(MessageTypes.BeginFile, assemblyDisplayName, null);
    client.SendMessage(MessageTypes.EndFile, assemblyDisplayName, null);
    loadedAssemblies[Marshal.GetTypeLibGuidForAssembly(returnAssembly)] 
      = returnAssembly;

    return returnAssembly;
  }
}