using System;
using DirectX;

public class NumberPicture : Picture
{
  // The left or right edge of the picture on the screen
  private int sideEdge;
  // The top edge of the picture on the screen
  private int topEdge;         
  // If true, sideEdge is the left edge. Otherwise, it is the right edge.
  private bool leftAligned;    
  // Width of each digit in the input bitmap, 0 to 9
  private int [] digitWidths;  
  // Amount of space between left edges of each digit in the input bitmap
  private int digitSpacing;
  // Space between digits in the output picture
  private int padding;         

  // Creates the picture from the bitmap
  public NumberPicture(string bitmapFile, bool hasTransparentColor,
    int [] digitWidths, int digitSpacing, int padding) 
    : base(bitmapFile, hasTransparentColor)
  {
    this.digitWidths = digitWidths;
    this.digitSpacing = digitSpacing;
    this.padding = padding;
  }

  // Sets the edges and alignment for the picture
  public void SetPosition(int sideEdge, int topEdge, bool leftAligned)
  {
    this.sideEdge = sideEdge;
    this.topEdge = topEdge;
    this.leftAligned = leftAligned;
  }

  // Draws the picture of the number passed in
  public void Draw(int number)
  {
    try
    {
      string stringNumber = number.ToString();
      int currentEdge = sideEdge;

      RECT r = new RECT();

      if (leftAligned)
      {
        for (int i = 0; i <= stringNumber.Length - 1; i++)
        {
          int digit = (int)Char.GetNumericValue(stringNumber[i]);

          r.Top = size.Top;
          r.Left = digitSpacing * digit;
          r.Bottom = size.Bottom;
          r.Right = r.Left + digitWidths[digit];

          Util.DrawSurface(dxSurface, currentEdge, topEdge, r,
            hasTransparentColor);

          currentEdge = currentEdge + digitWidths[digit] + padding;
        }
      }
      else
      {
        for (int i = stringNumber.Length - 1; i >= 0; i--)
        {
          int digit = (int)Char.GetNumericValue(stringNumber[i]);

          currentEdge = currentEdge - digitWidths[digit] - padding;

          r.Top = size.Top;
          r.Left = digitSpacing * digit;
          r.Bottom = size.Bottom;
          r.Right = r.Left + digitWidths[digit];

          Util.DrawSurface(dxSurface, currentEdge, topEdge, r, 
            hasTransparentColor);
        }
      }
    }
    catch
    {
      // If there's an error, just don't render a number.
    }
  }
}