using System;

public class Screen : IDisposable
{
  public const int Width = 640;  // Width of the screen in pixels
  public const int Height = 480; // Height of the screen in pixels
  protected Picture background;  // The background image

  // Creates the screen with the background picture
  public Screen(string bitmapFile)
  {
    background = new Picture(bitmapFile, false);
  }

  // Disposes the picture
  ~Screen()
  {
    background.Dispose();
  }

  // Disposes the picture
  public void Dispose()
  {
    // Since we're cleaning up now, suppress finalization
    background.Dispose();
    GC.SuppressFinalize(this);
  }

  // Draws the contents of the screen.
  // Overridding methods should not call the base implementation
  // since it calls Util.RenderComplete().
  public virtual void Render()
  {
    // In the base implementation, just draw the background
    // starting in the top left corner
    background.DrawAt(0, 0);

    // We're done rendering
    Util.RenderComplete();
  }
}