using System;
using System.Runtime.InteropServices;

namespace Win32
{
  internal delegate int WndProcDelegate(IntPtr hWnd, Messages msg,
    int wParam, int lParam);

  internal class WinMM
  {
    [DllImport("WinMM.dll")]
    internal static extern int timeGetTime();
  }

  internal class Kernel32
  {
    [DllImport("Kernel32.dll")]
    internal static extern bool QueryPerformanceCounter(
      out long lpPerformanceCount);

    [DllImport("Kernel32.dll")]
    internal static extern bool QueryPerformanceFrequency(
      out long lpFrequency);

    [DllImport("Kernel32.dll")]
    internal static extern IntPtr GetModuleHandle(String lpModuleName);
  }

  internal class User32
  {
    // Constants used with GetSystemMetrics
    internal const int SM_CXSCREEN = 0;
    internal const int SM_CYSCREEN = 1;

    // Constant used with LoadCursor
    internal const int IDC_ARROW = 32512;

    [DllImport("User32.dll")]
    internal static extern IntPtr RegisterClassEx(
      [In] ref WndClassEx rWndClassEx);

    [DllImport("User32.dll")]
    internal static extern IntPtr CreateWindowEx(int dwExStyle,
      String lpClassName, String lpWindowName, WindowStyles dwStyle,
      int x, int y, int nWidth, int nHeight, IntPtr hWndParent,
      IntPtr hMenu, IntPtr hInstance, IntPtr lpParam);

    [DllImport("User32.dll")]
    internal static extern bool ShowWindow(
      IntPtr hWnd, WindowState nCmdShow);

    [DllImport("User32.dll")]
    internal static extern bool UpdateWindow(IntPtr hWnd);

    [DllImport("User32.dll")]
    internal static extern bool DestroyWindow(IntPtr hWnd);

    [DllImport("User32.dll")]
    internal static extern bool PeekMessage(out Msg lpMsg, IntPtr hWnd,
      int wMsgFilterMin, int wMsgFilterMax, PeekMessages wRemoveMsg);

    [DllImport("User32.dll")]
    internal static extern int GetMessage(out Msg lpMsg, IntPtr hWnd,
      int wMsgFilterMin, int wMsgFilterMax);

    [DllImport("User32.dll")]
    internal static extern bool TranslateMessage(ref Msg lpMsg);

    [DllImport("User32.dll")]
    internal static extern int DispatchMessage(ref Msg lpMsg);

    [DllImport("User32.dll")]
    internal static extern void PostQuitMessage(int nExitCode);

    [DllImport("User32.dll")]
    internal static extern IntPtr LoadCursor(
      IntPtr hInstance, int lpCursorName);

    [DllImport("User32.dll")]
    internal static extern int ShowCursor(bool bShow);

    [DllImport("User32.dll")]
    internal static extern int GetSystemMetrics(int nIndex);

    [DllImport("User32.dll")]
    internal static extern int DefWindowProc(IntPtr hWnd, Messages msg,
      int wParam, int lParam);
  }

  internal class Gdi32
  {
    // Contant used with GetStockObject
    internal const int BLACK_BRUSH = 4;

    [DllImport("Gdi32.dll")]
    internal static extern IntPtr GetStockObject(int fnObject);
  }

  // Structs

  internal struct WndClassEx
  {
    internal int cbSize;
    internal ClassStyles style;
    internal WndProcDelegate lpfnWndProc;
    internal int cbClsExtra;
    internal int cbWndExtra;
    internal IntPtr hInstance;
    internal IntPtr hIcon;
    internal IntPtr hCursor;
    internal IntPtr hbrBackground;
    internal String lpszMenuName;
    internal String lpszClassName;
    internal IntPtr hIconSm;
  }

  internal struct Point
  {
    internal short x;
    internal short y;
  }

  internal struct Msg
  {
    internal IntPtr hwnd;
    internal Messages message;
    internal IntPtr wParam;
    internal IntPtr lParam;
    internal int time;
    internal Point pt;
  }

  // Enums

  internal enum ClassStyles : uint
  {
    CS_VREDRAW         = 0x0001,
    CS_HREDRAW         = 0x0002,
    CS_DBLCLKS         = 0x0008,
    CS_OWNDC           = 0x0020,
    CS_CLASSDC         = 0x0040,
    CS_PARENTDC        = 0x0080,
    CS_NOCLOSE         = 0x0200,
    CS_SAVEBITS        = 0x0800,
    CS_BYTEALIGNCLIENT = 0x1000,
    CS_BYTEALIGNWINDOW = 0x2000,
    CS_GLOBALCLASS     = 0x4000
  }

  internal enum WindowStyles : uint
  {
    WS_OVERLAPPED       = 0x00000000,
    WS_POPUP            = 0x80000000,
    WS_CHILD            = 0x40000000,
    WS_MINIMIZE         = 0x20000000,
    WS_VISIBLE          = 0x10000000,
    WS_DISABLED         = 0x08000000,
    WS_CLIPSIBLINGS     = 0x04000000,
    WS_CLIPCHILDREN     = 0x02000000,
    WS_MAXIMIZE         = 0x01000000,
    WS_CAPTION          = 0x00C00000,
    WS_BORDER           = 0x00800000,
    WS_DLGFRAME         = 0x00400000,
    WS_VSCROLL          = 0x00200000,
    WS_HSCROLL          = 0x00100000,
    WS_SYSMENU          = 0x00080000,
    WS_THICKFRAME       = 0x00040000,
    WS_GROUP            = 0x00020000,
    WS_TABSTOP          = 0x00010000,
    WS_MINIMIZEBOX      = 0x00020000,
    WS_MAXIMIZEBOX      = 0x00010000,
    WS_TILED            = WS_OVERLAPPED,
    WS_ICONIC           = WS_MINIMIZE,
    WS_SIZEBOX          = WS_THICKFRAME,
    WS_TILEDWINDOW      = WS_OVERLAPPEDWINDOW,
    WS_OVERLAPPEDWINDOW = WS_OVERLAPPED  |
                          WS_CAPTION     |
                          WS_SYSMENU     |
                          WS_THICKFRAME  |
                          WS_MINIMIZEBOX |
                          WS_MAXIMIZEBOX,
    WS_POPUPWINDOW      = WS_POPUP       |
                          WS_BORDER      |
                          WS_SYSMENU,
    WS_CHILDWINDOW      = WS_CHILD
  }

  internal enum WindowState
  {
    SW_HIDE            = 0,
    SW_SHOWNORMAL      = 1,
    SW_NORMAL          = 1,
    SW_SHOWMINIMIZED   = 2,
    SW_SHOWMAXIMIZED   = 3,
    SW_MAXIMIZE        = 3,
    SW_SHOWNOACTIVATE  = 4,
    SW_SHOW            = 5,
    SW_MINIMIZE        = 6,
    SW_SHOWMINNOACTIVE = 7,
    SW_SHOWNA          = 8,
    SW_RESTORE         = 9,
    SW_SHOWDEFAULT     = 10,
    SW_FORCEMINIMIZE   = 11
  }

  internal enum Messages : uint
  {
    WM_NULL           = 0x0000,
    WM_CREATE         = 0x0001,
    WM_DESTROY        = 0x0002,
    WM_MOVE           = 0x0003,
    WM_SIZE           = 0x0005,
    WM_ACTIVATE       = 0x0006,
    WM_SETFOCUS       = 0x0007,
    WM_KILLFOCUS      = 0x0008,
    WM_ENABLE         = 0x000A,
    WM_SETREDRAW      = 0x000B,
    WM_SETTEXT        = 0x000C,
    WM_GETTEXT        = 0x000D,
    WM_GETTEXTLENGTH  = 0x000E,
    WM_PAINT          = 0x000F,
    WM_CLOSE          = 0x0010,
    WM_QUIT           = 0x0012,
    WM_ERASEBKGND     = 0x0014,
    WM_SYSCOLORCHANGE = 0x0015,
    WM_SHOWWINDOW     = 0x0018,
    WM_WININICHANGE   = 0x001A,
    WM_SETTINGCHANGE  = WM_WININICHANGE,
    WM_DEVMODECHANGE  = 0x001B,
    WM_ACTIVATEAPP    = 0x001C,
    WM_FONTCHANGE     = 0x001D,
    WM_TIMECHANGE     = 0x001E,
    WM_CANCELMODE     = 0x001F,
    WM_SETCURSOR      = 0x0020,
    WM_MOUSEACTIVATE  = 0x0021,
    WM_CHILDACTIVATE  = 0x0022,
    WM_QUEUESYNC      = 0x0023,
    WM_GETMINMAXINFO  = 0x0024,
    WM_KEYFIRST       = 0x0100,
    WM_KEYDOWN        = 0x0100,
    WM_KEYUP          = 0x0101,
    WM_CHAR           = 0x0102,
    WM_DEADCHAR       = 0x0103,
    WM_SYSKEYDOWN     = 0x0104,
    WM_SYSKEYUP       = 0x0105,
    WM_SYSCHAR        = 0x0106,
    WM_SYSDEADCHAR    = 0x0107
  }

  internal enum PeekMessages : uint
  {
    PM_NOREMOVE = 0x0000,
    PM_REMOVE   = 0x0001,
    PM_NOYIELD  = 0x0002
  }

  // For brevity, this enumeration contains
  // only the key codes used in this application.
  internal enum Keys : uint
  {
    VK_PAUSE  = 0x13,
    VK_ESCAPE = 0x1B,
    VK_LEFT   = 0x25,
    VK_UP     = 0x26,
    VK_RIGHT  = 0x27,
    VK_DOWN   = 0x28
  }
}