using System;
using DirectX;

public class AnimatedEnvelopePicture : AnimatedPicture
{
  // Number of frames in the animation
  public const int NumFrames = 20;  
  // Frequency of the blinking of small envelopes
  private int blinkFrequency;       

  // Creates the picture from the bitmap.  
  // The bitmap contains each frame of the animation.
  public AnimatedEnvelopePicture(string bitmapFile) 
    : base(bitmapFile, true, NumFrames, 4, 0)
  {
    frameWidth = (size.Right - size.Left) / (framesInRow * 2);
    frameHeight = (size.Bottom - size.Top) / 5;
    blinkFrequency = 4;
  }

  // Draw the specified frame of the picture with the top left
  // corner at the (x,y) coordinate.  If infected is true, an
  // infected envelope is displayed.  The picture is clipped if part
  // of the picture is off the screen.
  public void DrawAt(int x, int y, int frameNumber, bool infected)
  {
    RECT frame = new RECT();

    // Choose the left edge of the "slice" of the input bitmap
    // based on whether or not we display an infected envelope.
    if (infected)
      frame.Left = ((frameNumber % framesInRow) * frameWidth) +
        frameWidth * framesInRow;
    else
      frame.Left = (frameNumber % framesInRow) * frameWidth;

    // Make the envelope flash when it's small.
    // This effects the top edge of the "slice".
    if (frameNumber >= 15 && ((Util.Counter / blinkFrequency) % 2 == 0))
      frame.Top = ((frameNumber / framesInRow) * frameHeight) 
        + frameHeight;
    else
      frame.Top = (frameNumber / framesInRow) * frameHeight;

    // The right and bottom edges are always 
    // a simple offset from the left and top
    frame.Right = frame.Left + frameWidth;
    frame.Bottom = frame.Top + frameHeight;

    RECT r = ClippedRectangle(ref x, ref y, frame);

    Util.DrawSurface(dxSurface, x, y, r, hasTransparentColor);
  }

  // This overload isn't supported on an AnimatedEnvelopePicture
  public override void DrawAt(int x, int y)
  {
    throw new NotSupportedException(
      "This action is not supported on an AnimatedEnvelopePicture.");
  }

  // This method isn't supported on an AnimatedEnvelopePicture
  public override void ResizeAndDrawAt(
    int x, int y, int width, int height)
  {
    throw new NotSupportedException(
      "This action is not supported on an AnimatedEnvelopePicture.");
  }
}