public class Character
{
  // # of pixels character moves horizontally
  public const int HorizontalJump = 80;  
  // # of pixels character moves vertically 
  public const int VerticalJump = 62;    

  // horizontal index of the starting envelope
  private const int InitialEnvelopeX = 7;  
  // vertical index of the starting envelope 
  private const int InitialEnvelopeY = 4;  

  // Current horizontal location in pixels
  public int LocationX;          
  // Current vertical location in pixels
  public int LocationY;          
  // true if the character is dying, false otherwise
  public bool IsDying;           

  // Number of frames since the character died
  private int framesSinceDeath;  
  // Horizontal index of the envelope under the character
  private int envelopeX;         
  // Vertical index of the envelope under the character
  private int envelopeY;         

  // Create the character and initialize its position
  public Character()
  {
    Reset();
  }

  // Re-initializes the character to the state when constructed
  public void Reset()
  {
    envelopeX = InitialEnvelopeX;
    envelopeY = InitialEnvelopeY;
    LocationX = Game.LeftSpace + InitialEnvelopeX * 
      (Game.EnvelopePicture.Width + Game.EnvelopePadding) +
      ((Game.EnvelopePicture.Width - Game.PlayerPicture.Width) / 2);
    LocationY = Game.TopSpace + (InitialEnvelopeY * 
      (Game.EnvelopePicture.Height + Game.EnvelopePadding)) - 
      ((Game.EnvelopePicture.Height - Game.PlayerPicture.Height) / 2)
      + (Game.EnvelopePadding / 2);
    IsDying = false;
    EnvelopeUnderneath.MakeVisibleAndInfected();
  }

  // Returns the envelope currently underneath the character
  public Envelope EnvelopeUnderneath
  {
    get { return Game.EnvelopeGrid[envelopeX, envelopeY]; }
  }

  // Moves the character.  Returns true if the move is valid,
  // false otherwise.
  public bool Move(Moves move)
  {
    switch (move)
    {
      case Moves.Down:
        if (LocationY + VerticalJump + Game.PlayerPicture.Height
          < Screen.Height)
        {
          LocationY += VerticalJump;
          envelopeY++;
          return true;
        }
        break;
      case Moves.Up:
        if (LocationY - HorizontalJump > 0)
        {
          LocationY -= VerticalJump;
          envelopeY--;
          return true;
        }
        break;
      case Moves.Left:
        if (LocationX - HorizontalJump + Game.PlayerPicture.Width > 0)
        {
          LocationX -= HorizontalJump;
          envelopeX--;
          if (envelopeX < 0) envelopeX = Game.GridWidth - 1;
          return true;
        }
        break;
      case Moves.Right:
        if (LocationX + HorizontalJump < Screen.Width)
        {
          LocationX += HorizontalJump;
          envelopeX = (envelopeX + 1) % Game.GridWidth;
          return true;
        }
        break;
      default:
        break;
    }
    return false;
  }

  // Draws the character at the current location.  If the character is
  // dying, draws a shrinking character and ends the current life
  public void Render()
  {
    if (IsDying)
    {
      framesSinceDeath++;

      // Make the character's image shrink
      Game.PlayerPicture.ResizeAndDrawAt(
        LocationX + framesSinceDeath,
        LocationY + (int)(framesSinceDeath * 1.5),
        Game.PlayerPicture.Width - (framesSinceDeath * 2),
        Game.PlayerPicture.Height - (framesSinceDeath * 2)
      );

      // If the character is finished dying, change the game's state
      if (Game.PlayerPicture.Width - (framesSinceDeath * 2) < 2 ||
        LocationX + Game.PlayerPicture.Width < 0)
        Game.State = GameStates.TryAgain;
    }
    else
    {
      Game.PlayerPicture.DrawAt(LocationX, LocationY);
    }
  }

  // Kills the character
  public void Kill()
  {
    IsDying = true;
    framesSinceDeath = 0;
  }
}