using System;
using System.Windows.Forms;
using Win32;

public class EmailAttackForm : Form
{
  private Game game;  // The game to play

  // Initializes and runs the game
  public EmailAttackForm()
  {
    long currentTime;       // Current time
    int timeBetweenFrames;  // Number of milliseconds between frames
    bool useBetterTimer;    // True if system has a performance counter
    long timeForNextFrame;  // Time to render the next frame

    Cursor.Hide();

    // Use a performance counter for higher precision timing, if available
    long frequency;
    if (Kernel32.QueryPerformanceFrequency(out frequency))
    {
      Kernel32.QueryPerformanceCounter(out timeForNextFrame);
      // Make the time between frames approx. 16 milliseconds
      timeBetweenFrames = (int)(frequency / 60);
      useBetterTimer = true;
    }
    else
    {
      timeForNextFrame = WinMM.timeGetTime();
      timeBetweenFrames = 16;
      useBetterTimer = false;
    }

    game = new Game(this.Handle);

    while (true)
    {
      if (useBetterTimer)
        Kernel32.QueryPerformanceCounter(out currentTime);
      else
        currentTime = WinMM.timeGetTime();

      if (currentTime > timeForNextFrame)
      {
        // Update the global counter.
        Util.Counter = (Util.Counter + 1) % 30000;

        Game.Render();

        // Check to see if the player died by
        // going off the left edge of the screen
        // or staying on an envelope that disappeared
        if (Game.State == GameStates.Running && !Game.Player.IsDying)
          Game.CheckForDeath();

        // Set the time for the next frame
        timeForNextFrame = currentTime + timeBetweenFrames;
      }
      Application.DoEvents();
    }
  }

  // Disposes the game
  protected override void Dispose(bool disposing)
  {
    if (disposing)
    {
	game.Dispose();
    }
    base.Dispose(disposing);
  }

  // Handles key presses by the user
  protected override void OnKeyDown(KeyEventArgs e)
  {
    if (Game.State == GameStates.Running)
    {
      // Pressing escape exits the game, pressing pause
      // pauses it, and pressing one of the arrow keys
      // moves the player
      switch (e.KeyCode)
      {
        case Keys.Down:
          Game.MovePlayer(Moves.Down);
          break;
        case Keys.Up:
          Game.MovePlayer(Moves.Up);
          break;
        case Keys.Left:
          Game.MovePlayer(Moves.Left);
          break;
        case Keys.Right:
          Game.MovePlayer(Moves.Right);
          break;
        case Keys.Escape:
          Environment.Exit(0);
          return;
        case Keys.Pause:
          Game.State = GameStates.Paused;
          break;
        default:
          break;
      }
    }
    else
    {
      // Pressing escape exits the game, and pressing
      // any other key starts it
      switch (e.KeyCode)
      {
        case Keys.Escape:
          Environment.Exit(0);
          return;
        default:
          Game.State = GameStates.Running;
          break;
      }
    }
    base.OnKeyDown(e);
  }

  // The entry point
  public static void Main()
  {
    EmailAttackForm eaf = new EmailAttackForm();
  }
}