using DirectX;
using System;
using System.Runtime.InteropServices;

public class Sound : IDisposable
{
  private DirectSoundBuffer buffer;  // The DirectX sound buffer

  // Creates the sound from the .WAV file
  public Sound(string waveFile)
  {
    try
    {
      buffer = Util.CreateSoundBufferFromFile(waveFile);
    }
    catch { // If we can't create the sound, silently fail
    }
  }

  // Disposes the sound.
  ~Sound()
  {
    Marshal.ReleaseComObject(buffer);
  }

  // Disposes the sound
  public void Dispose()
  {
    // Since we're cleaning up now, suppress finalization
    Marshal.ReleaseComObject(buffer);
    GC.SuppressFinalize(this);
  }

  // Plays the sound
  public void Play()
  {
    if (buffer != null)
      Util.PlayBuffer(buffer);
  }
}