// To compile:
// csc /t:library ColorfulVisualization.cs /r:System.Drawing.dll /r:System.Windows.Forms.dll /r:System.dll /r:BookExamples.WindowsMediaPlayer.dll

using System.Drawing;
using System.Windows.Forms;
using BookExamples.WindowsMediaPlayer;

public class ColorfulVisualization : Visualization
{
  private static Color backgroundColor = Color.LemonChiffon;

  public override string Title
  {
    get { return "Colorful"; }
  }

  public override string Description
  {
    get { return "A visualization whose background " +
                 "color can be configured."; 
    }
  }

  public override void DisplayPropertyPage(IWin32Window owner)
  {
    ColorDialog d = new ColorDialog();

    // We don't even use the owner window
    if (d.ShowDialog() == DialogResult.OK)
    {
      backgroundColor = d.Color;
    }
  }

  public override void Render(TimedLevel levels, 
    Graphics g, Rectangle r)
  {
    int y;        // The current Y-coordinate
    Pen pen;      // The pen used to draw on the background color
    Brush brush;  // The brush used to draw on the background color

    using (Brush b = new SolidBrush(backgroundColor))
    {
      // Fill background with the chosen color
      g.FillRectangle(b, r);
    }

    if (backgroundColor.GetBrightness() > 0.5)
    {
      pen = Pens.Black;
      brush = Brushes.Black;
    }
    else
    {
      pen = Pens.White;
      brush = Brushes.White; 
    }

    using (Font f = new Font("Arial", 14))
    {
      // Write the audio title at the top left corner (with some padding)
      g.DrawString(CurrentAudioTitle, f, brush, 10, 10);
    }

    // Walk through the frequencies until we run 
    // out of samples or drawing surface
    for (int x = r.Left; 
      x < r.Right && x < TimedLevel.MaxSamples; x++)
    {
      y = levels.GetScaledFrequency(AudioChannel.Left, x, r.Height);
      g.DrawLine(pen, x, r.Bottom, x, r.Bottom - y);
    }
  }
}