// This is to be compiled with a few other source files as follows:
//
// csc /t:library /out:BookExamples.WindowsMediaPlayer.dll IWMPEffects.cs SupportingTypes.cs Visualization.cs PresetEnumAttribute.cs /r:System.Drawing.dll /r:System.Windows.Forms.dll /r:System.dll

using System;
using System.Runtime.InteropServices;

namespace BookExamples.WindowsMediaPlayer
{
  [
    ComImport,
    Guid("D3984C13-C3CB-48E2-8BE5-5168340B4F35"),
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
  ]
  interface IWMPEffects
  {
    // Render using the rectangle on the normalized device context
    void Render(ref TimedLevel levels, IntPtr hdc, ref RECT r);

    // Provides the # of channels, sample rate and title of current audio
    void MediaInfo(int channelCount, int sampleRate, string title);

    // Provides the capabilities of the visualization
    VisualizationCapabilities Capabilities { get; }

    // Provides the display title of the visualization
    string Title { get; }

    // Provides the title for a preset
    string GetPresetTitle(int preset);

    // Provides the # of presets supported
    int PresetCount { get; }

    // Set/Get the current preset
    int CurrentPreset { set; get; }

    // Display the property page of the effect (if there is one)
    void DisplayPropertyPage(IntPtr hwndOwner);

    // Called when full screen rendering should start or stop
    void GoFullScreen(
      [MarshalAs(UnmanagedType.Bool)] bool startFullScreen);

    // This is called after a successful a call to GoFullScreen(true).
    // Return failure from this method to signal loss of full screen.
    void RenderFullScreen(ref TimedLevel levels);
  }
}