// This is to be compiled with a few other source files as follows:
//
// csc /t:library /out:BookExamples.WindowsMediaPlayer.dll IWMPEffects.cs SupportingTypes.cs Visualization.cs PresetEnumAttribute.cs /r:System.Drawing.dll /r:System.Windows.Forms.dll /r:System.dll

using System;
using System.Runtime.InteropServices;

namespace BookExamples.WindowsMediaPlayer
{
  // A standard Windows rectangle
  [ComVisible(false), StructLayout(LayoutKind.Sequential, Pack=4)]
  public struct RECT 
  {
    public int Left;
    public int Top;
    public int Right;
    public int Bottom;
    public override string ToString()
    {
      return "[" + Left + ", " + Top + "] - ["
        + Right + ", " + Bottom + "]";
    }
  }

  // A visualization's capabilities
  [ComVisible(false), Flags]
  public enum VisualizationCapabilities : uint
  {
    None = 0,
    CanGoFullScreen = 1,
    HasPropertyPage = 2,
    VariableFrequencyStep = 4,
  }

  // The state of the audio
  [ComVisible(false)]
  public enum AudioState       
  {
    Stopped = 0,
    Paused  = 1,
    Playing = 2
  }

  // The audio channels
  [ComVisible(false)]
  public enum AudioChannel
  {
    Left = 0,
    Right = 1
  }

  // The structure holding the frequency, waveform and 
  // state data crucial for rendering
  [ComVisible(false), StructLayout(LayoutKind.Sequential, Pack=4)]
  public struct TimedLevel
  {
    public const int MaxSamples = 1024;  

    // Beginning of the unmanaged definition

    [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2*MaxSamples)]
    private byte [] frequency;      

    [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2*MaxSamples)]
    private byte [] waveform;       

    public AudioState State;
    public long TimeStamp;

    // End of the unmanaged definition

    // Returns the frequency (0-255) at the specified channel and index
    public byte GetFrequency(AudioChannel channel, int index)
    {
      if (index < 0) index = 0;
      return frequency[(int)channel * MaxSamples + index];
    }

    // Returns the frequency at the specified channel and index,
    // scaled between 0 and maxValue
    public int GetScaledFrequency(
      AudioChannel channel, int index, int maxValue)
    {
      return (int)(GetFrequency(channel, index) * (maxValue / 256f));
    }

    // Returns the waveform value (0-255) 
    // at the specified channel and index
    public byte GetWaveform(AudioChannel channel, int index)
    {
      if (index < 0) index = 0;
      return waveform[(int)channel * MaxSamples + index];
    }

    // Returns the waveform value at the specified channel and index,
    // scaled between 0 and maxValue
    public int GetScaledWaveform(
      AudioChannel channel, int index, int maxValue)
    {
      return (int)(GetWaveform(channel, index) * (maxValue / 256f));
    }
  }	
}