// To compile:
// csc /t:library WizardVisualization.cs /r:System.Drawing.dll /r:BookExamples.WindowsMediaPlayer.dll

using System.Drawing;
using BookExamples.WindowsMediaPlayer;

// The presets supported by this visualization
enum Presets
{
  Bars,
  Scope
}

[PresetEnum(typeof(Presets))]
public class WizardVisualization : Visualization
{
  public override string Title
  {
    get { return "Wizard"; }
  }

  public override string Description
  {
    get { return "A .NET visualization based on the one " + 
      "generated by the Windows Media Visualization Wizard."; }
  }

  public override void Render(TimedLevel levels,
    Graphics g, Rectangle r)
  {
    int y;      // The current Y-coordinate
    int lastY;  // Stores the previous Y-coordinate for the scope preset

    // Fill background with black
    g.FillRectangle(Brushes.Black, r);

    // Draw based on the current preset
    switch ((Presets)CurrentPreset)
    {
      case Presets.Bars:
      {
        // Walk through the frequencies until we run
        // out of samples or drawing surface.
        for (int x = r.Left; 
          x < r.Right && x < TimedLevel.MaxSamples; x++)
        {
          y = levels.GetScaledFrequency(AudioChannel.Left, x, r.Height);
          g.DrawLine(Pens.MintCream, x, r.Bottom, 
            x, r.Bottom - y);
        }
      }
      break;

      case Presets.Scope:
      {
        y = levels.GetScaledWaveform(AudioChannel.Left, r.Left, 
          r.Height);
        lastY = y;

        // Walk through the waveform data until we run 
        // out of samples or drawing surface.
        for (int x = r.Left;
          x < r.Right && x < TimedLevel.MaxSamples; x++)
        {
          g.DrawLine(Pens.Tomato, x - 1, lastY, x, y);
          y = levels.GetScaledWaveform(AudioChannel.Left, x, r.Height);
          lastY = y;
        }
      }
      break;
    }
  }
}